@@ -1,8 +1,3 @@
-/*
- * $Id: axis.h,v 1.166.2.5 2017/10/09 01:06:33 sfeam Exp $
- *
- */
-
 /*[
  * Copyright 2000, 2004   Thomas Williams, Colin Kelley
  *
@@ -36,17 +31,10 @@
 #ifndef GNUPLOT_AXIS_H
 #define GNUPLOT_AXIS_H
 
-#ifdef DISABLE_NONLINEAR_AXES
-# define nonlinear(axis) FALSE
-# define invalid_coordinate(x,y) FALSE
-#else
-
-# define NONLINEAR_AXES 1
+/* Aug 2017 - unconditional support for nonlinear axes */
 # define nonlinear(axis) ((axis)->linked_to_primary != NULL && (axis)->link_udf->at != NULL)
 # define invalid_coordinate(x,y) ((unsigned)(x)==intNaN || (unsigned)(y)==intNaN)
-#endif
 
-#include <stddef.h>		/* for offsetof() */
 #include "gp_types.h"		/* for TBOOLEAN */
 
 #include "gadgets.h"
@@ -65,9 +53,6 @@
  * SECOND_X_AXIS = FIRST_X_AXIS + SECOND_AXES
  * FIRST_X_AXIS & SECOND_AXES == 0
  */
-#ifndef MAX_PARALLEL_AXES
-# define MAX_PARALLEL_AXES MAXDATACOLS-1
-#endif
 
 typedef enum AXIS_INDEX {
     NO_AXIS = -2,
@@ -87,15 +72,16 @@ typedef enum AXIS_INDEX {
     T_AXIS,
     U_AXIS,
     V_AXIS,		/* Last index into axis_array[] */
-    PARALLEL_AXES,	/* Parallel axis data is allocated dynamically */
+    PARALLEL_AXES,	/* Parallel axis structures are allocated dynamically */
     THETA_index = 1234,	/* Used to identify THETA_AXIS */
+    MOUSE_PROXY_AXIS,	/* Used by watchpoints code */
 
     AXIS_ARRAY_SIZE = PARALLEL_AXES
 } AXIS_INDEX;
 
 
 /* sample axis doesn't need mtics, so use the slot to hold sample interval */
-# define SAMPLE_INTERVAL mtic_freq
+# define SAMPLE_INTERVAL miniticscale
 
 /* What kind of ticmarking is wanted? */
 typedef enum en_ticseries_type {
@@ -135,19 +121,20 @@ typedef struct ticdef {
     TBOOLEAN rangelimited;		/* Limit tics to data range */
     TBOOLEAN enhanced;			/* Use enhanced text mode or labels */
     TBOOLEAN logscaling;		/* place tics using old logscale algorithm */
+    TBOOLEAN force_linear_tics;		/* override the logscaling flag */
 } t_ticdef;
 
-/* we want two auto modes for minitics - default where minitics are
- * auto for log/time and off for linear, and auto where auto for all
- * graphs I've done them in this order so that logscale-mode can
- * simply test bit 0 to see if it must do the minitics automatically.
- * similarly, conventional plot can test bit 1 to see if minitics are
- * required */
+/* Version 5.5 (Jun 2021)
+ * Default is AUTO for log scale, OFF otherwise
+ * New mode TIME only applies when axis or axis tics are set to time mode
+ * and the user has requested "set mxtics time <N> <units>".
+ */
 typedef enum en_minitics_status {
     MINI_OFF,
     MINI_DEFAULT,
     MINI_USER,
-    MINI_AUTO
+    MINI_AUTO,
+    MINI_TIME
 } t_minitics_status;
 
 /* Values to put in the axis_tics[] variables that decides where the
@@ -189,9 +176,9 @@ typedef enum e_constraint {
 /* The unit the tics of a given time/date axis are to interpreted in */
 /* HBB 20040318: start at one, to avoid undershoot */
 typedef enum e_timelevel {
+    TIMELEVEL_DEFAULT = 0,
     TIMELEVEL_SECONDS = 1, TIMELEVEL_MINUTES, TIMELEVEL_HOURS,
-    TIMELEVEL_DAYS, TIMELEVEL_WEEKS, TIMELEVEL_MONTHS,
-    TIMELEVEL_YEARS
+    TIMELEVEL_DAYS, TIMELEVEL_WEEKS, TIMELEVEL_MONTHS, TIMELEVEL_YEARS
 } t_timelevel;
 
 typedef struct axis {
@@ -239,13 +226,14 @@ typedef struct axis {
 
 /* ticmark control variables */
     int ticmode;		/* tics on border/axis? mirrored? */
-    struct ticdef ticdef;	/* tic series definition */
-    int tic_rotate;		/* ticmarks rotated by this angle */
     enum JUSTIFY tic_pos;	/* left/center/right tic label justification */
+    struct ticdef ticdef;	/* tic series definition */
+    float tic_rotate;		/* ticmarks rotated by this angle */
     TBOOLEAN gridmajor;		/* Grid lines wanted on major tics? */
     TBOOLEAN gridminor;		/* Grid lines for minor tics? */
-    t_minitics_status minitics;	/* minor tic mode (none/auto/user)? */
-    double mtic_freq;		/* minitic stepsize */
+    t_minitics_status minitics;	/* minor tic mode (none/auto/user/time) */
+    t_timelevel minitic_units;	/* in time mode, DAYS/MONTHS/etc */
+    int mtic_freq;		/* number of minitics per interval */
     double ticscale;		/* scale factor for tic marks (was (0..1])*/
     double miniticscale;	/* and for minitics */
     double ticstep;		/* increment used to generate tic placement */
@@ -254,6 +242,7 @@ typedef struct axis {
 /* time/date axis control */
     td_type datatype;		/* {DT_NORMAL|DT_TIMEDATE} controls _input_ */
     td_type tictype;		/* {DT_NORMAL|DT_TIMEDATE|DT_DMS} controls _output_ */
+    t_timelevel tic_units;	/* granularity of auto-generated time tics */
     char *formatstring;		/* the format string for output */
     char *ticfmt;		/* autogenerated alternative to formatstring (needed??) */
     t_timelevel timelevel;	/* minimum time unit used to quantize ticks */
@@ -265,9 +254,10 @@ typedef struct axis {
     text_label label;		/* label string and position offsets */
     TBOOLEAN manual_justify;	/* override automatic justification */
     lp_style_type *zeroaxis;	/* usually points to default_axis_zeroaxis */
+    double paxis_x;		/* x coordinate of parallel axis */
 } AXIS;
 
-#define DEFAULT_AXIS_TICDEF {TIC_COMPUTED, NULL, {TC_DEFAULT, 0, 0.0}, {NULL, {0.,0.,0.}, FALSE},  { character, character, character, 0., 0., 0. }, FALSE, TRUE, FALSE }
+#define DEFAULT_AXIS_TICDEF {TIC_COMPUTED, NULL, {TC_DEFAULT, 0, 0.0}, {NULL, {0.,0.,0.}, FALSE},  { character, character, character, 0., 0., 0. }, FALSE, TRUE, FALSE, FALSE }
 #define DEFAULT_AXIS_ZEROAXIS {0, LT_AXIS, 0, DASHTYPE_AXIS, 0, 0, 1.0, PTSZ_DEFAULT, DEFAULT_P_CHAR, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}
 
 #define DEFAULT_AXIS_STRUCT {						    \
@@ -286,12 +276,14 @@ typedef struct axis {
 	NULL, NULL,		/* linked_to_primary, linked_to_secondary */\
 	NULL,      		/* link function */                         \
 	NO_TICS,		/* tic output positions (border, mirror) */ \
+	CENTRE,		 	/* horizontal justification */		    \
 	DEFAULT_AXIS_TICDEF,	/* tic series definition */		    \
-	0, CENTRE,	 	/* tic_rotate, horizontal justification */  \
+	0.0,			/* tic_rotate */			    \
 	FALSE, FALSE,	 	/* grid{major,minor} */			    \
-	MINI_DEFAULT, 10.,	/* minitics, mtic_freq */		    \
+	MINI_DEFAULT, 1, 10,	/* minitics, units, mtic_freq */	    \
 	1.0, 0.5, 0.0, TRUE,	/* ticscale, miniticscale, ticstep, tic_in */ \
 	DT_NORMAL, DT_NORMAL,	/* datatype for input, output */	    \
+	TIMELEVEL_SECONDS,	/* default granularity (seconds) for tics */\
 	NULL, NULL,      	/* output format, another output format */  \
 	0,			/* timelevel */                             \
 	0,			/* index (e.g.FIRST_Y_AXIS) */		    \
@@ -319,8 +311,8 @@ extern AXIS *shadow_axis_array;
 extern const AXIS_DEFAULTS axis_defaults[AXIS_ARRAY_SIZE];
 extern const AXIS default_axis_state;
 
-/* EAM DEBUG - Dynamic allocation of parallel axes. */
-extern AXIS *parallel_axis;
+/* Dynamic allocation of parallel axis structures */
+extern AXIS *parallel_axis_array;
 extern int num_parallel_axes;
 
 /* A parsing table for mapping axis names into axis indices. For use
@@ -353,6 +345,12 @@ extern int grid_layer;
 /* Whether to draw the axis tic labels and tic marks in front of everything else */
 extern TBOOLEAN grid_tics_in_front;
 
+/* Whether to draw vertical grid lines in 3D */
+extern TBOOLEAN grid_vertical_lines;
+
+/* Whether to draw a grid in spiderplots */
+extern TBOOLEAN grid_spiderweb;
+
 /* Whether or not to draw a separate polar axis in polar mode */
 extern TBOOLEAN raxis;
 
@@ -367,14 +365,16 @@ extern int tic_text, rotate_tics, tic_hjust, tic_vjust;
 extern struct lp_style_type grid_lp; /* linestyle for major grid lines */
 extern struct lp_style_type mgrid_lp; /* linestyle for minor grid lines */
 
-extern double polar_grid_angle; /* angle step in polar grid in radians */
+extern double theta_grid_angle; /* angle step in polar grid in radians */
 extern double theta_origin;	/* 0 = right side of plot */
 extern double theta_direction;	/* 1 = counterclockwise -1 = clockwise */
 
-/* Length of the longest tics label, set by widest_tic_callback(): */
+/* Length of the longest tics label, set by widest_tic_callback() */
 extern int widest_tic_strlen;
 
-/* flag to indicate that in-line axis ranges should be ignored */
+/* flag to indicate that in-line axis ranges should be ignored
+ * and zoom/pan range limits take precedence over auto-scaling
+ */
 extern TBOOLEAN inside_zoom;
 
 /* axes being used by the current plot */
@@ -393,92 +393,18 @@ extern struct axis THETA_AXIS;
 /* -------- macros using these variables: */
 
 /* Macros to map from user to terminal coordinates and back */
-#define AXIS_MAP(axis, variable)		\
-  (int) ((axis_array[axis].term_lower)		\
-	 + ((variable) - axis_array[axis].min)	\
-	 * axis_array[axis].term_scale + 0.5)
-#define AXIS_MAPBACK(axis, pos)						   \
-  (((double)(pos)-axis_array[axis].term_lower)/axis_array[axis].term_scale \
-   + axis_array[axis].min)
+#define AXIS_MAP(axis, variable)        axis_map(        &axis_array[axis], variable)
+#define AXIS_MAP_DOUBLE(axis, variable) axis_map_double( &axis_array[axis], variable)
+#define AXIS_MAPBACK(axis, pos)         axis_mapback(    &axis_array[axis], pos)
 
 /* Same thing except that "axis" is a pointer, not an index */
-#define axis_map(axis, variable)		\
-    (int) ((axis)->term_lower + ((variable) - (axis)->min) * (axis)->term_scale + 0.5)
+#define axis_map_double(axis, variable)         \
+    ((axis)->term_lower + ((variable) - (axis)->min) * (axis)->term_scale)
+#define axis_map_toint(x) (int)( (x) + 0.5 )
+#define axis_map(axis, variable) axis_map_toint( axis_map_double(axis, variable) )
+
 #define axis_mapback(axis, pos) \
-    (((double)(pos) - axis->term_lower)/axis->term_scale + axis->min)
-
-#if defined(NONLINEAR_AXES) && (NONLINEAR_AXES > 0)
-
-/* These become no-ops because the nonlinear axis code stores untransformed values */
-#define AXIS_DO_LOG(axis,value) (value)
-#define AXIS_UNDO_LOG(axis,value) (value)
-#define axis_do_log(axis,value) (value)
-#define axis_undo_log(axis,value) (value)
-#define AXIS_LOG_VALUE(axis,value) (value)
-#define AXIS_DE_LOG_VALUE(axis,coordinate) (coordinate)
-#define axis_log_value(axis,value) (value)
-#define axis_de_log_value(axis,coordinate) (coordinate)
-
-#else /* !(NONLINEAR_AXES > 0) */
-
-/* Macros to deal with logscale axis values being stored as log */
-#define AXIS_DO_LOG(axis,value) (log(value) / axis_array[axis].log_base)
-#define AXIS_UNDO_LOG(axis,value) exp((value) * axis_array[axis].log_base)
-#define axis_do_log(axis,value) (log(value) / axis->log_base)
-#define axis_undo_log(axis,value) exp((value) * axis->log_base)
-
-/* Same, but these test if the axis is log, first: */
-#define AXIS_LOG_VALUE(axis,value)				\
-    (axis_array[axis].log ? AXIS_DO_LOG(axis,value) : (value))
-#define AXIS_DE_LOG_VALUE(axis,coordinate)				  \
-    (axis_array[axis].log ? AXIS_UNDO_LOG(axis,coordinate): (coordinate))
-#define axis_log_value(axis,value)				\
-    (axis->log ? axis_do_log(axis,value) : (value))
-#define axis_de_log_value(axis,coordinate)				  \
-    (axis->log ? axis_undo_log(axis,coordinate): (coordinate))
-
-#endif /* (NONLINEAR_AXES > 0) */
-
-/* April 2015:  I'm not 100% sure, but I believe there is no longer
- * any need to treat 2D and 3D axis initialization differently
- */
-#define AXIS_INIT3D(axis, islog_override, infinite) \
-        axis_init((&axis_array[axis]), infinite)
-
-#define AXIS_INIT2D(axis, infinite) \
-        axis_init((&axis_array[axis]), infinite)
-
-/* AXIS_INIT2D_REFRESH and AXIS_UPDATE2D_REFRESH(axis) are for volatile data */
-#define AXIS_INIT2D_REFRESH(axis, infinite)				\
-do {									\
-    AXIS *this = axis_array + axis;					\
-									\
-    this->autoscale = this->set_autoscale;				\
-    this->min = (infinite && (this->set_autoscale & AUTOSCALE_MIN))	\
-	? VERYLARGE : AXIS_LOG_VALUE(axis, this->set_min);		\
-    this->max = (infinite && (this->set_autoscale & AUTOSCALE_MAX))	\
-	? -VERYLARGE : AXIS_LOG_VALUE(axis, this->set_max);		\
-    this->log_base = this->log ? log(this->base) : 0;			\
-} while(0)
-
-#define AXIS_UPDATE2D_REFRESH(axis)					\
-do {									\
-    AXIS *this_axis = axis_array + axis;				\
-    if ((this_axis->set_autoscale & AUTOSCALE_MIN) == 0)		\
-	this_axis->min = AXIS_LOG_VALUE(axis, this_axis->set_min);	\
-    if ((this_axis->set_autoscale & AUTOSCALE_MAX) == 0)		\
-	this_axis->max = AXIS_LOG_VALUE(axis, this_axis->set_max);	\
-} while (0)
-
-/* Simplest form of autoscaling (no check on autoscale constraints).
- * Used by refresh_bounds() and refresh_3dbounds().
- */
-#define autoscale_one_point(axis, x) do {\
-    if (axis->set_autoscale & AUTOSCALE_MIN && x < axis->min) \
-	axis->min = x; \
-    if (axis->set_autoscale & AUTOSCALE_MAX && x > axis->max) \
-	axis->max = x; \
-    } while (0);
+    (((double)(pos) - (axis)->term_lower)/(axis)->term_scale + (axis)->min)
 
 /* parse a position of the form
  *    [coords] x, [coords] y {,[coords] z}
@@ -493,122 +419,29 @@ do {									\
     (store) = get_num_or_time(this_axis);				\
 } while(0)
 
-/* store VALUE or log(VALUE) in STORE, set TYPE as appropriate
- * Do OUT_ACTION or UNDEF_ACTION as appropriate
- * adjust range provided type is INRANGE (ie dont adjust y if x is outrange
- * VALUE must not be same as STORE
- * NOAUTOSCALE is per-plot property, whereas AUTOSCALE_XXX is per-axis.
- * Note: see the particular implementation for COLOR AXIS below.
- */
-#if defined(NONLINEAR_AXES) && (NONLINEAR_AXES > 0)
-#define OPTIMIZE_LOG 1
-#else
-#define OPTIMIZE_LOG 0
-#endif
 
-#define ACTUAL_STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, TYPE, AXIS,  \
-					       NOAUTOSCALE, OUT_ACTION,   \
-					       UNDEF_ACTION)              \
-do {									  \
-    struct axis *axis = AXIS;						  \
-    double curval = (VALUE);						  \
-    /* Version 5: OK to store infinities or NaN */			  \
-    STORE = curval;							  \
-    if (! (curval > -VERYLARGE && curval < VERYLARGE)) {		  \
-	TYPE = UNDEFINED;						  \
-	UNDEF_ACTION;							  \
-	break;								  \
-    }									  \
-    if (axis->log) {							  \
-	if (curval < 0.0) {						  \
-	    if (!OPTIMIZE_LOG) STORE = not_a_number();			  \
-	    TYPE = UNDEFINED;						  \
-	    UNDEF_ACTION;						  \
-	    break;							  \
-	} else if (curval == 0.0) {					  \
-	    if (!OPTIMIZE_LOG) STORE = -VERYLARGE;			  \
-	    TYPE = OUTRANGE;						  \
-	    OUT_ACTION;							  \
-	    break;							  \
-	} else if (!OPTIMIZE_LOG) {					  \
-	    STORE = axis_do_log(axis, curval);				  \
-	}								  \
-    }									  \
-    if (NOAUTOSCALE)							  \
-	break;  /* this plot is not being used for autoscaling */	  \
-    if (TYPE != INRANGE)						  \
-	break;  /* don't set y range if x is outrange, for example */	  \
-    if (   (curval < axis->min)						  \
-        && ((curval <= axis->max) || (axis->max == -VERYLARGE))		  \
-       ) {								  \
-	if (axis->autoscale & AUTOSCALE_MIN)	{			  \
-	    if (axis->min_constraint & CONSTRAINT_LOWER) {		  \
-		if (axis->min_lb <= curval) {				  \
-		    axis->min = curval;					  \
-		} else {						  \
-		    axis->min = axis->min_lb;				  \
-		    TYPE = OUTRANGE;					  \
-		    OUT_ACTION;						  \
-		    break;						  \
-		}							  \
-	    } else {							  \
-		axis->min = curval;					  \
-	    }								  \
-	} else if (curval != axis->max) {				  \
-	    TYPE = OUTRANGE;						  \
-	    OUT_ACTION;							  \
-	    break;							  \
-	}								  \
-    }									  \
-    if ( curval > axis->max						  \
-    &&  (curval >= axis->min || axis->min == VERYLARGE)) {		  \
-	if (axis->autoscale & AUTOSCALE_MAX)	{			  \
-	    if (axis->max_constraint & CONSTRAINT_UPPER) {		  \
-		if (axis->max_ub >= curval) {		 		  \
-		    axis->max = curval;					  \
-		} else {						  \
-		    axis->max = axis->max_ub;				  \
-		    TYPE =OUTRANGE;					  \
-		    OUT_ACTION;						  \
-		    break;						  \
-		}							  \
-	    } else {							  \
-		axis->max = curval;					  \
-	    }								  \
-	} else if (curval != axis->min) {				  \
-	    TYPE = OUTRANGE;						  \
-	    OUT_ACTION;							  \
-	}								  \
-    }									  \
-    /* Only update data min/max if the point is INRANGE Jun 2016 */	  \
-    if (TYPE == INRANGE) { 						  \
-	if (axis->data_min > curval)					  \
-	    axis->data_min = curval;					  \
-	if (axis->data_max < curval)					  \
-	    axis->data_max = curval;					  \
-    }									  \
+/*
+ * Gradually replacing extremely complex macro ACTUAL_STORE_AND_UPDATE_RANGE
+ * (called 50+ times) with a subroutine. The original logic was that in-line
+ * code was faster than calls to a subroutine, but on current hardware it is
+ * better to have one cached copy than to have 50 separate uncached copies.
+ *
+ * The difference between STORE_AND_UPDATE_RANGE and store_and_update_range
+ * is that the former takes an axis index and the latter an axis pointer.
+ */
+#define STORE_AND_UPDATE_RANGE(STORE, VALUE, TYPE, AXIS, NOAUTOSCALE, UNDEF_ACTION)	 \
+ if (AXIS != NO_AXIS) do { \
+    if (store_and_update_range( &(STORE), VALUE, &(TYPE), (&axis_array[AXIS]), NOAUTOSCALE) \
+        == UNDEFINED) { \
+	UNDEF_ACTION; \
+    } \
 } while(0)
 
-/* normal calls go though this macro */
-#define STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, TYPE, AXIS, NOAUTOSCALE, OUT_ACTION, UNDEF_ACTION)	 \
- if (AXIS != NO_AXIS) \
- ACTUAL_STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, TYPE, (&axis_array[AXIS]), NOAUTOSCALE, OUT_ACTION, UNDEF_ACTION)
-
-/* Implementation of the above for the color axis. It should not change
- * the type of the point (out-of-range color is plotted with the color
- * of the min or max color value).
- */
-#define COLOR_STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, TYPE, AXIS,	  \
-			       NOAUTOSCALE, OUT_ACTION, UNDEF_ACTION)	  \
-{									  \
-    coord_type c_type_tmp = TYPE;					  \
-    ACTUAL_STORE_WITH_LOG_AND_UPDATE_RANGE(STORE, VALUE, c_type_tmp, &axis_array[AXIS],	  \
-					   NOAUTOSCALE, OUT_ACTION, UNDEF_ACTION); \
-}
-
-/* #define NOOP (0) caused many warnings from gcc 3.2 */
+/* Use NOOP for UNDEF_ACTION if no action is wanted */
 #define NOOP ((void)0)
 
+
+
 /* HBB 20000506: new macro to automatically build initializer lists
  * for arrays of AXIS_ARRAY_SIZE=11 equal elements */
 #define AXIS_ARRAY_INITIALIZER(value) {			\
@@ -622,71 +455,87 @@ do {									  \
 #define CheckZero(x,tic) (fabs(x) < ((tic) * SIGNIF) ? 0.0 : (x))
 
 /* Function pointer type for callback functions to generate ticmarks */
-typedef void (*tic_callback) __PROTO((struct axis *, double, char *, int, 
-				struct lp_style_type, struct ticmark *));
+typedef void (*tic_callback) (struct axis *, double, char *, int, 
+			struct lp_style_type, struct ticmark *);
 
 /* ------------ functions exported by axis.c */
-t_autoscale load_range __PROTO((struct axis *, double *, double *, t_autoscale));
-void axis_unlog_interval __PROTO((struct axis *, double *, double *, TBOOLEAN));
-void axis_invert_if_requested __PROTO((struct axis *));
-void axis_revert_range __PROTO((AXIS_INDEX));
-void axis_revert_and_unlog_range __PROTO((AXIS_INDEX));
-void axis_init __PROTO((AXIS *this_axis, TBOOLEAN infinite));
-void set_explicit_range __PROTO((struct axis *axis, double newmin, double newmax));
-double axis_log_value_checked __PROTO((AXIS_INDEX, double, const char *));
-void axis_checked_extend_empty_range __PROTO((AXIS_INDEX, const char *mesg));
-void axis_check_empty_nonlinear __PROTO((struct axis *this_axis));
-char * copy_or_invent_formatstring __PROTO((struct axis *));
-double quantize_normal_tics __PROTO((double, int));
-void setup_tics __PROTO((struct axis *, int));
-void gen_tics __PROTO((struct axis *, tic_callback));
-void axis_output_tics __PROTO((AXIS_INDEX, int *, AXIS_INDEX, tic_callback));
-void axis_set_scale_and_range __PROTO((struct axis *axis, int lower, int upper));
-void axis_draw_2d_zeroaxis __PROTO((AXIS_INDEX, AXIS_INDEX));
-TBOOLEAN some_grid_selected __PROTO((void));
-void add_tic_user __PROTO((struct axis *, char *, double, int));
-double get_num_or_time __PROTO((struct axis *));
-TBOOLEAN bad_axis_range __PROTO((struct axis *axis));
+coord_type store_and_update_range(double *store, double curval, coord_type *type,
+				struct axis *axis, TBOOLEAN noautoscale);
+void autoscale_one_point( struct axis *axis, double x );
+t_autoscale load_range(struct axis *, double *, double *, t_autoscale);
+void check_log_limits(struct axis *, double, double);
+void axis_invert_if_requested(struct axis *);
+void axis_check_range(AXIS_INDEX);
+void axis_init(AXIS *this_axis, TBOOLEAN infinite);
+void set_explicit_range(struct axis *axis, double newmin, double newmax);
+double axis_log_value_checked(AXIS_INDEX, double, const char *);
+void axis_checked_extend_empty_range(AXIS_INDEX, const char *mesg);
+void axis_check_empty_nonlinear(struct axis *this_axis);
+char * copy_or_invent_formatstring(struct axis *);
+double quantize_normal_tics(double, int);
+void setup_tics(struct axis *, int);
+void gen_tics(struct axis *, tic_callback);
+void axis_output_tics(AXIS_INDEX, int *, AXIS_INDEX, tic_callback);
+void axis_set_scale_and_range(struct axis *axis, int lower, int upper);
+void axis_draw_2d_zeroaxis(AXIS_INDEX, AXIS_INDEX);
+TBOOLEAN some_grid_selected(void);
+void add_tic_user(struct axis *, char *, double, int);
+double get_num_or_time(struct axis *);
+TBOOLEAN bad_axis_range(struct axis *axis);
 
-void save_writeback_all_axes __PROTO((void));
-int  parse_range __PROTO((AXIS_INDEX axis));
-void parse_skip_range __PROTO((void));
-void check_axis_reversed __PROTO((AXIS_INDEX axis));
+void save_writeback_all_axes(void);
+int  parse_range(AXIS_INDEX axis);
+void parse_skip_range(void);
+void check_axis_reversed(AXIS_INDEX axis);
+double parse_one_range_limit( double default_value );
 
 /* set widest_tic_label: length of the longest tics label */
-void widest_tic_callback __PROTO((struct axis *, double place, char *text, int ticlevel,
-			struct lp_style_type grid, struct ticmark *));
+void widest_tic_callback(struct axis *, double place, char *text, int ticlevel,
+			struct lp_style_type grid, struct ticmark *);
+/* callback that just counts the number of tics that are placed */
+void tic_count_callback(struct axis *, double place, char *text, int ticlevel,
+			struct lp_style_type grid, struct ticmark *);
+void sanity_check_log_tics( int axis_index );
 
-void get_position __PROTO((struct position *pos));
-void get_position_default __PROTO((struct position *pos, enum position_type default_type, int ndim));
+void get_position(struct position *pos);
+void get_position_default(struct position *pos, enum position_type default_type,
+			TBOOLEAN clear, int ndim);
 
-void gstrdms __PROTO((char *label, char *format, double value));
+void gstrdms(char *label, char *format, double value);
 
-void clone_linked_axes __PROTO((AXIS *axis1, AXIS *axis2));
-AXIS *get_shadow_axis __PROTO((AXIS *axis));
-void extend_primary_ticrange __PROTO((AXIS *axis));
-void update_primary_axis_range __PROTO((struct axis *secondary));
-void update_secondary_axis_range __PROTO((struct axis *primary));
-void reconcile_linked_axes __PROTO((AXIS *primary, AXIS *secondary));
+void clone_linked_axes(AXIS *axis1, AXIS *axis2);
+AXIS *get_shadow_axis(AXIS *axis);
+void extend_primary_ticrange(AXIS *axis);
+void update_primary_axis_range(struct axis *secondary);
+void update_secondary_axis_range(struct axis *primary);
+void reconcile_linked_axes(AXIS *primary, AXIS *secondary);
+void extend_autoscaled_log_axis(AXIS *primary);
 
-int map_x __PROTO((double value));
-int map_y __PROTO((double value));
+int map_x(double value);
+int map_y(double value);
+int map_ysharp(double value);
 
-coord_type polar_to_xy __PROTO(( double theta, double r, double *x, double *y, TBOOLEAN update));
-double polar_radius __PROTO((double r));
+double map_x_double(double value);
+double map_y_double(double value);
 
-void set_cbminmax __PROTO((void));
+coord_type polar_to_xy( double theta, double r, double *x, double *y, TBOOLEAN update);
+double polar_radius(double r);
 
-void save_autoscaled_ranges __PROTO((AXIS *, AXIS *));
-void restore_autoscaled_ranges __PROTO((AXIS *, AXIS *));
+void set_cbminmax(void);
 
-char * axis_name __PROTO((AXIS_INDEX));
-void init_sample_range __PROTO((AXIS *axis));
-void init_parallel_axis __PROTO((AXIS *, AXIS_INDEX));
-AXIS * extend_parallel_axis __PROTO((int ));
+void save_autoscaled_ranges(AXIS *, AXIS *);
+void restore_autoscaled_ranges(AXIS *, AXIS *);
+
+char * axis_name(AXIS_INDEX);
+void init_sample_range(AXIS *axis, enum PLOT_TYPE plot_type);
+void init_parallel_axis(AXIS *, AXIS_INDEX);
+void extend_parallel_axis(int);
 
 /* Evaluate the function linking a secondary axis to its primary axis */
-double eval_link_function __PROTO((AXIS *, double));
+double eval_link_function(AXIS *, double);
+
+/* For debugging */
+void dump_axis_range(AXIS *axis);
 
 /* macro for tic scale, used in all tic_callback functions */
 #define tic_scale(ticlevel, axis) \
