@@ -674,24 +674,9 @@ cbtick_callback(
 	term_apply_lp_properties(&border_lp);	/* border linetype */
     }
 
-    if (len != 0) {
-	/* draw tic */
-	(*term->move) (x1, y1);
-	(*term->vector) (x2, y2);
-
-	/* draw mirrored tic */
-	if (this_axis->ticmode & TICS_MIRROR) {
-	    if (color_box.rotation == 'h') {
-		y1 = color_box.bounds.ytop;
-		y2 = color_box.bounds.ytop + len;
-	    } else {
-		x1 = color_box.bounds.xleft;
-		x2 = color_box.bounds.xleft - len;
-	    }
-	    (*term->move) (x1, y1);
-	    (*term->vector) (x2, y2);
-	}
-    }
+    /* draw tic */
+    (*term->move) (x1, y1);
+    (*term->vector) (x2, y2);
 
     /* draw label */
     if (text) {
@@ -746,6 +731,18 @@ cbtick_callback(
 	term_apply_lp_properties(&border_lp);	/* border linetype */
     }
 
+    /* draw tic on the mirror side */
+    if (this_axis->ticmode & TICS_MIRROR) {
+	if (color_box.rotation == 'h') {
+	    y1 = color_box.bounds.ytop;
+	    y2 = color_box.bounds.ytop + len;
+	} else {
+	    x1 = color_box.bounds.xleft;
+	    x2 = color_box.bounds.xleft - len;
+	}
+	(*term->move) (x1, y1);
+	(*term->vector) (x2, y2);
+    }
 }
 
 /*
@@ -978,6 +975,13 @@ f_palette(union argument *arg)
     if (((CB_AXIS.set_autoscale & AUTOSCALE_BOTH) != 0)
     && (fabs(CB_AXIS.min) >= VERYLARGE || fabs(CB_AXIS.max) >= VERYLARGE))
 	int_error(NO_CARET, "palette(z) requires known cbrange");
+    else {
+	/* Needed if the palette is to be queried before an actual plot */
+	if ((CB_AXIS.set_autoscale & AUTOSCALE_MIN) == 0)
+	    CB_AXIS.min = CB_AXIS.set_min;
+	if ((CB_AXIS.set_autoscale & AUTOSCALE_MAX) == 0)
+	    CB_AXIS.max = CB_AXIS.set_max;
+    }
     rgb = rgb_from_gray(cb2gray(z));
 
     push(Ginteger(&result, rgb));
@@ -1304,6 +1308,9 @@ set_palette_file()
     if (!(name_str = string_or_express(NULL)))
 	int_error(c_token, "expecting filename or datablock");
 
+    /* Same interlock as plot/splot/stats */
+    inside_plot_command = TRUE;
+
     df_set_plot_mode(MODE_QUERY);	/* Needed only for binary datafiles */
     df_open(name_str, 4, NULL);
 
@@ -1361,6 +1368,7 @@ set_palette_file()
     df_close();
     if (i==0)
 	int_error( c_token, "No valid palette found" );
+    inside_plot_command = FALSE;
 
     sm_palette.gradient_num = i;
     check_palette_grayscale();
