@@ -1,7 +1,3 @@
-/*
- * $Id: fit.h,v 1.31 2015/02/26 18:43:58 sfeam Exp $
- */
-
 /* GNUPLOT - fit.h */
 
 /*  NOTICE: Change of Copyright Status
@@ -38,6 +34,7 @@
 
 /* #if... / #include / #define collection: */
 
+#include <setjmp.h>
 #include "syscfg.h"
 #include "stdfn.h"
 #include "gp_types.h"
@@ -55,11 +52,14 @@
 /* Type definitions */
 
 typedef enum e_verbosity_level {
-    QUIET, RESULTS, BRIEF, VERBOSE
+    QUIET=1, RESULTS, BRIEF, VERBOSE
 } verbosity_level;
 
 typedef char fixstr[MAX_ID_LEN+1];
 
+/* pointer to longjmp recovery point of "fit" command */
+extern JMP_BUF *fit_env;
+
 /* Exported Variables of fit.c */
 
 extern const char *FITLIMIT;
@@ -79,6 +79,7 @@ extern double epsilon_abs;  /* absolute convergence criterion */
 extern int maxiter;
 extern int fit_wrap;
 extern TBOOLEAN fit_v4compatible;
+extern char *last_fit_command;
 
 /* Prototypes of functions exported by fit.c */
 
@@ -89,13 +90,12 @@ __declspec(noreturn) void error_ex(int t_num, const char *str, ...);
 #else
 void error_ex(int t_num, const char *str, ...);
 #endif
-void init_fit __PROTO((void));
-void update __PROTO((char *pfile, char *npfile));
-void save_fit __PROTO((FILE *fp));
-void fit_command __PROTO((void));
-size_t wri_to_fil_last_fit_cmd __PROTO((FILE *fp));
-char *getfitlogfile __PROTO((void));
-char *getfitscript __PROTO((void));
+void init_fit(void);
+void update(char *pfile, char *npfile);
+void save_fit(FILE *fp);
+void fit_command(void);
+char *getfitlogfile(void);
+const char *getfitscript(void);
 
 void call_gnuplot(const double *par, double *data);
 TBOOLEAN regress_check_stop(int iter, double chisq, double last_chisq, double lambda);
