@@ -5,4 +5,7 @@ void f_faddeeva(union argument *z);
 void f_voigtp(union argument *z);
 void f_voigt(union argument *z);
 void f_erfi(union argument *z);
+void f_VP_fwhm(union argument *z);
+void f_FresnelC(union argument *z);
+void f_FresnelS(union argument *z);
 #endif
