@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: plot.c,v 1.164.2.1 2014/12/31 04:32:09 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - plot.c */
 
 /*[
@@ -42,57 +38,36 @@ static char *RCSid() { return RCSid("$Id: plot.c,v 1.164.2.1 2014/12/31 04:32:09
 #include "eval.h"
 #include "fit.h"
 #include "gp_hist.h"
+#include "gplocale.h"
+#include "loadpath.h"
 #include "misc.h"
 #include "readline.h"
 #include "setshow.h"
 #include "term_api.h"
 #include "util.h"
-#include "variable.h"
 #include "version.h"
+#include "voxelgrid.h"
+#include "encoding.h"
+#include "xdg.h"
 
 #include <signal.h>
 #include <setjmp.h>
 
-#ifdef OS2 /* os2.h required for gpexecute.h */
-# define INCL_DOS
+#ifdef OS2
 # define INCL_REXXSAA
-# ifdef OS2_IPC
-#  define INCL_DOSSEMAPHORES
-# endif
 # include <os2.h>
 #endif /* OS2 */
 
-#ifdef USE_MOUSE
-# include "mouse.h" /* for mouse_setting */
-# include "gpexecute.h"
-#endif
-
-/* on OS/2 this is needed even without USE_MOUSE */
-#if defined(OS2_IPC) && !defined(USE_MOUSE)
-# include "gpexecute.h"
-#endif
-
 #if defined(MSDOS) || defined(__EMX__) || (defined(WGP_CONSOLE) && defined(MSVC))
 # include <io.h>
 #endif
 
-#ifdef VMS
-# ifndef __GNUC__
-#  include <unixio.h>
-# endif
-# include <smgdef.h>
-# include <ssdef.h>
-extern int vms_vkid;
-extern smg$create_virtual_keyboard();
-extern int vms_ktid;
-extern smg$create_key_table();
-#endif /* VMS */
-
-#ifdef _Windows
+#ifdef _WIN32
 # include <windows.h>
 # include "win/winmain.h"
 # include "win/wcommon.h"
-#endif /* _Windows */
+# include <io.h>           /* for isatty */
+#endif /* _WIN32 */
 
 /* GNU readline
  * Only required by two files directly,
@@ -106,14 +81,15 @@ extern smg$create_key_table();
 /* BSD editline
 */
 #ifdef HAVE_LIBEDITLINE
-# include <editline/readline.h>
+#  include <editline/readline.h>
+#  include <histedit.h>
 #endif
 
 /* enable gnuplot history with readline */
 #ifdef GNUPLOT_HISTORY
-# ifndef GNUPLOT_HISTORY_FILE
-#  define GNUPLOT_HISTORY_FILE "~/.gnuplot_history"
-# endif
+#  ifndef GNUPLOT_HISTORY_FILE
+#    define GNUPLOT_HISTORY_FILE "~/.gnuplot_history"
+#  endif
 /*
  * expanded_history_filename points to the value from 'tilde_expand()',
  * which expands '~' to the user's home directory, or $HOME.
@@ -122,15 +98,16 @@ extern smg$create_key_table();
  */
 static char *expanded_history_filename;
 
-static void wrapper_for_write_history __PROTO((void));
+static void wrapper_for_write_history(void);
 
 #endif				/* GNUPLOT_HISTORY */
 
 TBOOLEAN interactive = TRUE;	/* FALSE if stdin not a terminal */
 TBOOLEAN noinputfiles = TRUE;	/* FALSE if there are script files */
-TBOOLEAN reading_from_dash=FALSE; /* True if processing "-" as an input file */
-TBOOLEAN skip_gnuplotrc = FALSE;/* skip system gnuplotrc and ~/.gnuplot */
-TBOOLEAN persist_cl = FALSE; /* TRUE if -persist is parsed in the command line */
+TBOOLEAN reading_from_dash=FALSE;	/* True if processing "-" as an input file */
+TBOOLEAN skip_gnuplotrc = FALSE;	/* skip system gnuplotrc and ~/.gnuplot */
+TBOOLEAN persist_cl = FALSE; 		/* --persist command line option */
+TBOOLEAN slow_font_startup = FALSE;	/* --slow command line option */
 
 /* user home directory */
 static const char *user_homedir = NULL;
@@ -138,29 +115,32 @@ static const char *user_homedir = NULL;
 /* user shell */
 const char *user_shell = NULL;
 
-static TBOOLEAN successful_initialization = FALSE;
+/* not static because unset.c refers to it when in debugging mode */
+TBOOLEAN successful_initialization = FALSE;
 
 #ifdef X11
-extern int X11_args __PROTO((int, char **)); /* FIXME: defined in term/x11.trm */
+extern int X11_args(int, char **); /* FIXME: defined in term/x11.trm */
 #endif
 
 /* patch to get home dir, see command.c */
-#ifdef DJGPP
-# include <dir.h>               /* MAXPATH */
-char HelpFile[MAXPATH];
-#endif /*   - DJL */
+#if defined(MSDOS) || defined(OS2)
+char HelpFile[PATH_MAX];
+static char progpath[PATH_MAX] = "";
+#endif
 
 /* a longjmp buffer to get back to the command line */
 static JMP_BUF command_line_env;
 
-static void load_rcfile __PROTO((int where));
-static RETSIGTYPE inter __PROTO((int anint));
-static void init_memory __PROTO((void));
+static void load_rcfile(int where);
+static RETSIGTYPE inter(int anint);
+static void init_memory(void);
 
 static int exit_status = EXIT_SUCCESS;
 
 /* Flag for asynchronous handling of Ctrl-C. Used by fit.c and Windows */
 TBOOLEAN ctrlc_flag = FALSE;
+/* Flag for (asynchronous) term signal on Windows. */
+TBOOLEAN terminate_flag = FALSE;
 
 #ifdef OS2
 # include <process.h>
@@ -171,7 +151,7 @@ TBOOLEAN CallFromRexx = FALSE;
 static RETSIGTYPE
 inter(int anint)
 {
-    (void) anint;		/* aovid -Wunused warning */
+    (void) anint;		/* avoid -Wunused warning */
     (void) signal(SIGINT, (sigfunc) inter);
     (void) signal(SIGFPE, SIG_DFL);	/* turn off FPE trapping */
 
@@ -202,106 +182,76 @@ inter(int anint)
 #endif
 }
 
-#ifdef LINUXVGA
-/* utility functions to ensure that setuid gnuplot
- * assumes root privileges only for those parts
- * of the code which require root rights.
+/* Normally we return to the command line at a point equivalent
+ * to program entry, so most context is lost.  All loops and
+ * loaded command files are exited; open files are closed, etc.
  *
- * By "Dr. Werner Fink" <werner@suse.de>
+ * The exception to this is when an error is encountered during a
+ * "fit" command.  In this case the program returns to the same
+ * point it would have reached if the fit command had succeeded,
+ * possibly inside a loop or a loaded command file.
  */
-static uid_t euid, ruid;
-static gid_t egid, rgid;
-static int asked_privi = 0;
-
-void
-drop_privilege()
-{
-    if (!asked_privi) {
-	euid = geteuid();
-	egid = getegid();
-	ruid = getuid();
-	rgid = getgid();
-	asked_privi = 1;
-    }
-    if (setegid(rgid) == -1)
-	(void) fprintf(stderr, "setegid(%d): %s\n",
-		       (int) rgid, strerror(errno));
-    if (seteuid(ruid) == -1)
-	(void) fprintf(stderr, "seteuid(%d): %s\n",
-		       (int) ruid, strerror(errno));
-}
-
-void
-take_privilege()
-{
-    if (!asked_privi) {
-	euid = geteuid();
-	egid = getegid();
-	ruid = getuid();
-	rgid = getgid();
-	asked_privi = 1;
-    }
-    if (setegid(egid) == -1)
-	(void) fprintf(stderr, "setegid(%d): %s\n",
-		       (int) egid, strerror(errno));
-    if (seteuid(euid) == -1)
-	(void) fprintf(stderr, "seteuid(%d): %s\n",
-		       (int) euid, strerror(errno));
-}
-
-#endif /* LINUXVGA */
-
-/* a wrapper for longjmp so we can keep everything local */
 void
 bail_to_command_line()
 {
-#ifdef _Windows
+#ifdef _WIN32
     kill_pending_Pause_dialog();
     ctrlc_flag = FALSE;
 #endif
-    LONGJMP(command_line_env, TRUE);
+    if (fit_env)
+	LONGJMP(*fit_env, TRUE);
+#if defined (WXWIDGETS) && defined (HAVE_GTK)
+    else if (wxt_event_processing) {
+	extern JMP_BUF *wxt_env;
+	LONGJMP(*wxt_env, TRUE);
+    }
+#endif
+    else
+	LONGJMP(command_line_env, TRUE);
 }
 
-#if defined(_Windows)
+#if defined(_WIN32)
 int
-gnu_main(int argc, char **argv)
+gnu_main(int argc_orig, char **argv)
 #else
 int
-main(int argc, char **argv)
+main(int argc_orig, char **argv)
 #endif
 {
     int i;
 
-#ifdef LINUXVGA
-    LINUX_setup();		/* setup VGA before dropping privilege DBT 4/5/99 */
-    drop_privilege();
-#endif
+    /* We want the current value of argc to persist across a LONGJMP from int_error().
+     * Without this the compiler may put it on the stack, which LONGJMP clobbers.
+     * Here we try make it a volatile variable that optimization will not affect.
+     * Why do we not have to do the same for argv?   I don't know.
+     * But the test cases that broke with generic argc seem fine with generic argv.
+     */
+    static volatile int argc;
+    argc = argc_orig;
+
 /* make sure that we really have revoked root access, this might happen if
    gnuplot is compiled without vga support but is installed suid by mistake */
 #ifdef __linux__
-    setuid(getuid());
+    if (setuid(getuid()) != 0) {
+	fprintf(stderr,"gnuplot: refusing to run at elevated privilege\n");
+	exit(EXIT_FAILURE);
+    }
 #endif
 
-#if defined(MSDOS) && !defined(_Windows) && !defined(__GNUC__)
-    PC_setup();
-#endif /* MSDOS !Windows */
-
 /* HBB: Seems this isn't needed any more for DJGPP V2? */
 /* HBB: disable all floating point exceptions, just keep running... */
 #if defined(DJGPP) && (DJGPP!=2)
     _control87(MCW_EM, MCW_EM);
 #endif
 
-#if defined(OS2)
-    int rc;
-#ifdef OS2_IPC
-    char semInputReadyName[40];
-    sprintf( semInputReadyName, "\\SEM32\\GP%i_Input_Ready", getpid() );
-    rc = DosCreateEventSem(semInputReadyName,&semInputReady,0,0);
-    if (rc != 0)
-      fputs("DosCreateEventSem error\n",stderr);
-#endif
-    rc = RexxRegisterSubcomExe("GNUPLOT", (PFN) RexxInterface, NULL);
+#ifdef OS2
+    {
+	int rc = RexxRegisterSubcomExe("GNUPLOT", (PFN) RexxInterface, NULL);
+	(void) rc;
+# ifdef OS2_IPC
+	os2_ipc_setup();
+# endif
+    }
 #endif
 
 /* malloc large blocks, otherwise problems with fragmented mem */
@@ -309,47 +259,86 @@ main(int argc, char **argv)
     malloc_debug(7);
 #endif
 
-/* get helpfile from home directory */
-#ifdef __DJGPP__
+
+/* init progpath and get helpfile from executable directory */
+#if defined(MSDOS) || defined(OS2)
     {
 	char *s;
-	strcpy(HelpFile, argv[0]);
-	for (s = HelpFile; *s; s++)
-	    if (*s == DIRSEP1)
-		*s = DIRSEP2;	/* '\\' to '/' */
-	strcpy(strrchr(HelpFile, DIRSEP2), "/gnuplot.gih");
-    }			/* Add also some "paranoid" tests for '\\':  AP */
-#endif /* DJGPP */
 
-#ifdef VMS
-    unsigned int status[2] = { 1, 0 };
+#ifdef __EMX__
+	_execname(progpath, sizeof(progpath));
+#else
+	safe_strncpy(progpath, argv[0], sizeof(progpath));
 #endif
+	/* convert '/' to '\\' */
+	for (s = progpath; *s != NUL; s++)
+	    if (*s == DIRSEP2)
+		*s = DIRSEP1;
+	/* cut program name */
+	s = strrchr(progpath, DIRSEP1);
+	if (s != NULL)
+	    s++;
+	else
+	    s = progpath;
+	*s = NUL;
+	/* init HelpFile */
+	strcpy(HelpFile, progpath);
+	strcat(HelpFile, "gnuplot.gih");
+	/* remove trailing "bin/" from progpath */
+	if ((s != NULL) && (s - progpath >= 4)) {
+	    s -= 4;
+	    if (strncasecmp(s, "bin", 3) == 0)
+		*s = NUL;
+	}
+    }
+#endif /* MSDOS || OS2 */
 
-#if defined(HAVE_LIBEDITLINE)
+#if (defined(PIPE_IPC) || defined(_WIN32)) && (defined(HAVE_LIBREADLINE) || (defined(HAVE_LIBEDITLINE)))
+    /* Editline needs this to be set before the very first call to readline(). */
     rl_getc_function = getc_wrapper;
 #endif
 
-#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
+#if defined(HAVE_LIBREADLINE)
     /* T.Walter 1999-06-24: 'rl_readline_name' must be this fix name.
      * It is used to parse a 'gnuplot' specific section in '~/.inputrc'
      * or gnuplot specific commands in '.editrc' (when using editline
      * instead of readline) */
     rl_readline_name = "Gnuplot";
     rl_terminal_name = getenv("TERM");
-#if defined(HAVE_LIBREADLINE)
     using_history();
-#else
-    history_init();
-#endif
-#endif
-#if defined(HAVE_LIBREADLINE) && !defined(MISSING_RL_TILDE_EXPANSION)
+
+    #if !defined(MISSING_RL_TILDE_EXPANSION)
     rl_complete_with_tilde_expansion = 1;
+    #endif
+
+    #if defined(RL_VERSION_MAJOR)
+    /* Starting with readline v8.1 bracketed paste mode is on by default.
+     * This breaks multi-line pasted input to gnuplot because it looks like
+     * one long run-on line.
+     */
+    if (RL_VERSION_MAJOR >= 8)
+	rl_variable_bind ("enable-bracketed-paste", "off");
+    #endif
+#endif
+
+#if defined(HAVE_LIBEDITLINE)
+    rl_readline_name = "Gnuplot";
+    rl_terminal_name = getenv("TERM");
+    el = el_init("gnuplot", stdin, stdout, stderr);
+    history_init();
 #endif
 
     for (i = 1; i < argc; i++) {
 	if (!argv[i])
 	    continue;
 
+	if (!strcmp(argv[i], "-c")) {
+	    /* The rest of the command line is a scriptfile and its arguments.
+	     * Do not try to interpret them here (ignore -V, --help, etc).
+	     */
+	    break;
+	}
+
 	if (!strcmp(argv[i], "-V") || !strcmp(argv[i], "--version")) {
 	    printf("gnuplot %s patchlevel %s\n",
 		    gnuplot_version, gnuplot_patchlevel);
@@ -357,12 +346,10 @@ main(int argc, char **argv)
 
 	} else if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help")) {
 	    printf( "Usage: gnuplot [OPTION] ... [FILE]\n"
-#ifdef X11
-		    "for X11 options see 'help X11->command-line-options'\n"
-#endif
 		    "  -V, --version\n"
 		    "  -h, --help\n"
 		    "  -p  --persist\n"
+		    "  -s  --slow\n"
 		    "  -d  --default-settings\n"
 		    "  -c  scriptfile ARG1 ARG2 ... \n"
 		    "  -e  \"command1; command2; ...\"\n"
@@ -381,26 +368,19 @@ main(int argc, char **argv)
 	    return 0;
 
 	} else if (!strncmp(argv[i], "-persist", 2) || !strcmp(argv[i], "--persist")
-#ifdef _Windows
+#ifdef _WIN32
 		|| !stricmp(argv[i], "-noend") || !stricmp(argv[i], "/noend")
 #endif
 		) {
 	    persist_cl = TRUE;
+	} else if (!strncmp(argv[i], "-slow", 2) || !strcmp(argv[i], "--slow")) {
+	    slow_font_startup = TRUE;
 	} else if (!strncmp(argv[i], "-d", 2) || !strcmp(argv[i], "--default-settings")) {
 	    /* Skip local customization read from ~/.gnuplot */
 	    skip_gnuplotrc = TRUE;
 	}
     }
 
-#ifdef X11
-    /* the X11 terminal removes tokens that it recognizes from argv. */
-    {
-	int n = X11_args(argc, argv);
-	argv += n;
-	argc -= n;
-    }
-#endif
-
     setbuf(stderr, (char *) NULL);
 
 #ifdef HAVE_SETVBUF
@@ -418,7 +398,6 @@ main(int argc, char **argv)
      * EAM - Jan 2013 YES.
      */
     setvbuf(stdin, (char *) NULL, _IONBF, 0);
-
 #endif
 
     gpoutfile = stdout;
@@ -426,6 +405,7 @@ main(int argc, char **argv)
     /* Initialize pre-loaded user variables */
     /* "pi" is hard-wired as the first variable */
     (void) add_udv_by_name("GNUTERM");
+    (void) add_udv_by_name("I");
     (void) add_udv_by_name("NaN");
     init_constants();
     udv_user_head = &(udv_NaN->next_udv);
@@ -433,15 +413,11 @@ main(int argc, char **argv)
     init_memory();
 
     interactive = FALSE;
-    init_terminal();		/* can set term type if it likes */
-    push_terminal(0);		/* remember the default terminal */
 
-    /* reset the terminal when exiting */
-    /* this is done through gp_atexit so that other terminal functions
-     * can be registered to be executed before the terminal is reset. */
-    gp_atexit(term_reset);
+    /* April 2017:  We used to call init_terminal() here, but now   */
+    /* We defer initialization until error handling has been set up. */
 
-# if defined(WIN32) && !defined(WGP_CONSOLE)
+# if defined(_WIN32) && !defined(WGP_CONSOLE)
     interactive = TRUE;
 # else
     interactive = isatty(fileno(stdin));
@@ -454,7 +430,7 @@ main(int argc, char **argv)
      * command line arguments are file names or an explicit in-line "-e command".
      */
     for (i = 1; i < argc; i++) {
-# ifdef _Windows
+# ifdef _WIN32
 	if (!stricmp(argv[i], "/noend"))
 	    continue;
 # endif
@@ -464,95 +440,133 @@ main(int argc, char **argv)
 	}
     }
 
-    /* Need this before show_version is called for the first time */
-
-    if (interactive)
-	show_version(stderr);
-    else
-	show_version(NULL); /* Only load GPVAL_COMPILE_OPTIONS */
-
-#ifdef WGP_CONSOLE
-#ifdef CONSOLE_SWITCH_CP
-    if (cp_changed && interactive) {
-	fprintf(stderr,
-	    "\ngnuplot changed the codepage of this console from %i to %i to\n" \
-	    "match the graph window. Some characters might only display correctly\n" \
-	    "if you change the font to a non-raster type.\n",
-	    cp_input, GetConsoleCP());
-    }
-#else
-    if ((GetConsoleCP() != GetACP()) && interactive) {
-	fprintf(stderr,
-	    "\nWarning: The codepage of the graph window (%i) and that of the\n" \
-	    "console (%i) differ. Use `set encoding` or `!chcp` if extended\n" \
-	    "characters don't display correctly.\n",
-	    GetACP(), GetConsoleCP());
-    }
-#endif
-#endif
-
-    update_gpval_variables(3);  /* update GPVAL_ variables available to user */
-
 #ifdef VMS
-    /* initialise screen management routines for command recall */
-    if (status[1] = smg$create_virtual_keyboard(&vms_vkid) != SS$_NORMAL)
-	done(status[1]);
-    if (status[1] = smg$create_key_table(&vms_ktid) != SS$_NORMAL)
-	done(status[1]);
+    vms_init_screen();
 #endif /* VMS */
 
     if (!SETJMP(command_line_env, 1)) {
 	/* first time */
 	interrupt_setup();
-	/* should move this stuff to another initialisation routine,
-	 * something like init_set() maybe */
 	get_user_env();
 	init_loadpath();
 	init_locale();
 
 	memset(&sm_palette, 0, sizeof(sm_palette));
 	init_fit();		/* Initialization of fitting module */
+#ifdef READLINE
+	/* When using the built-in readline, we set the initial
+	   encoding according to the locale as this is required
+	   to properly handle keyboard input. */
+	init_encoding();
+#endif
+	init_gadgets();
+
+	/* Now that error handling is in place, it is safe to parse GNUTERM
+	 * during terminal initialization.  However if terminal initialization
+	 * exits via int_error(), the session and history state initialization
+	 * can only be reached via a GOTO from the error return.
+	 */
+	init_terminal();
+	push_terminal(0);	/* remember the initial terminal */
+
+	/* atexit processing is done in reverse order. We want
+	 * the generic terminal shutdown in term_reset to be executed before
+	 * any terminal specific cleanup requested by individual terminals.
+	 */
+	gp_atexit(term_reset);
+
+#ifdef X11
+	/* the X11 terminal removes tokens that it recognizes from argv. */
+	if (term && !strcmp(term->name, "x11")) {
+	    int n = X11_args(argc, argv);
+	    argv += n;
+	    argc -= n;
+	}
+#endif
+	FALLBACK_ONCE_FROM_INIT_FAILURE:
+
+	/* Version 6: defer splash page until after initialization */
+	if (interactive)
+	    show_version(stderr);
+	else
+	    show_version(NULL); /* Only load GPVAL_COMPILE_OPTIONS */
+
+	/* update GPVAL_ variables available to user */
+	update_gpval_variables(3);
+
+	/* Execute commands in ~/.gnuplot */
 	init_session();
 
+	if (interactive)
+	   fprintf(stderr, "\n\tTerminal type is now %s\n", term->name);
+
 	if (interactive && term != 0) {		/* not unknown */
 #ifdef GNUPLOT_HISTORY
-#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
+#if (defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)) && !defined(_WIN32)
 	    expanded_history_filename = tilde_expand(GNUPLOT_HISTORY_FILE);
 #else
 	    expanded_history_filename = gp_strdup(GNUPLOT_HISTORY_FILE);
 	    gp_expand_tilde(&expanded_history_filename);
 #endif
-	    read_history(expanded_history_filename);
+	    if (read_history(expanded_history_filename)) {
+#ifdef USE_XDG_BASEDIR
+		/* use $XDG_STATE_DIR/gnuplot_history only if
+		 * $XDG_STATE_DIR exists or can be created */
+		char *xdgpath = xdg_get_path(kXDGStateHome, "gnuplot_history",
+				FALSE /* no subdir */, TRUE /* create */);
+		if (xdgpath) {
+		    free(expanded_history_filename);
+		    expanded_history_filename = xdgpath;
+		    read_history(expanded_history_filename);
+		}
+#endif /* USE_XDG_BASEDIR */
+	    }
 
 	    /*
 	     * It is safe to ignore the return values of 'atexit()' and
 	     * 'on_exit()'. In the worst case, there is no history of your
-	     * currrent session and you have to type all again in your next
+	     * current session and you have to type it all again in your next
 	     * session.
 	     */
 	    gp_atexit(wrapper_for_write_history);
 #endif /* GNUPLOT_HISTORY */
 
-	    fprintf(stderr, "\nTerminal type set to '%s'\n", term->name);
+#if defined(READLINE) && defined(WGP_CONSOLE)
+	    fprintf(stderr, "Encoding set to '%s'.\n", encoding_names[encoding]);
+#endif
 	}			/* if (interactive && term != 0) */
     } else {
 	/* come back here from int_error() */
 	if (!successful_initialization) {
-	    /* Only print the warning once */
+	    /* Only warn and attempt a fall-back on the first failure.
+	     * There is no guarantee that we will come out of this in a
+	     * usable state, but the only truly safe alternative would be
+	     * to give up and exit the program.
+	     */
 	    successful_initialization = TRUE;
-	    fprintf(stderr,"WARNING: Error during initialization\n\n");
+	    fprintf(stderr, "WARNING: Error during initialization\n");
+	    fprintf(stderr, "         Check initialization files and environment variables (e.g. GNUTERM)\n");
+	    change_term("unknown", 7);
+	    goto FALLBACK_ONCE_FROM_INIT_FAILURE;
 	}
 	if (interactive == FALSE)
 	    exit_status = EXIT_FAILURE;
 #ifdef HAVE_READLINE_RESET
-	else
-	{
+	else {
 	    /* reset properly readline after a SIGINT+longjmp */
 	    rl_reset_after_signal ();
 	}
 #endif
 
-	load_file_error();	/* if we were in load_file(), cleanup */
+	/* If we were in a "load", "call", or nested bracketed clause 
+	 * clean up the lf_push/pop stack and restore shadowed variables.
+	 */
+	clause_reset_after_error();
+	lf_reset_after_error();
+
+	/* We are certainly no longer in a plot command */
+	inside_plot_command = FALSE;
+
 	SET_CURSOR_ARROW;
 
 #ifdef VMS
@@ -577,8 +591,6 @@ main(int argc, char **argv)
 	/* entering the loop again from the top finds them messed up.  */
 	/* If we reenter the loop via a goto then there is some hope   */
 	/* that code reordering does not hurt us.                      */
-	/* NB: the test for interactive means that execution from a    */
-	/* pipe will not continue after an error. Do we want this?     */
 	if (reading_from_dash && interactive)
 	    goto RECOVER_FROM_ERROR_IN_DASH;
 	reading_from_dash = FALSE;
@@ -594,13 +606,13 @@ main(int argc, char **argv)
 	    ++argv;
 	    c_token = 0;
 	    if (!strncmp(*argv, "-persist", 2) || !strcmp(*argv, "--persist")
-#ifdef _Windows
+#ifdef _WIN32
 		|| !stricmp(*argv, "-noend") || !stricmp(*argv, "/noend")
 #endif
 	    ) {
 		FPRINTF((stderr,"'persist' command line option recognized\n"));
 	    } else if (strcmp(*argv, "-") == 0) {
-#if defined(_Windows) && !defined(WGP_CONSOLE)
+#if defined(_WIN32) && !defined(WGP_CONSOLE)
 		TextShow(&textwin);
 		interactive = TRUE;
 #else
@@ -612,8 +624,10 @@ RECOVER_FROM_ERROR_IN_DASH:
 		while (!com_line());
 		reading_from_dash = FALSE;
 		interactive = FALSE;
+		noinputfiles = FALSE;
 
 	    } else if (strcmp(*argv, "-e") == 0) {
+		int save_state = interactive;
 		--argc; ++argv;
 		if (argc <= 0) {
 		    fprintf(stderr, "syntax:  gnuplot -e \"commands\"\n");
@@ -622,6 +636,10 @@ RECOVER_FROM_ERROR_IN_DASH:
 		interactive = FALSE;
 		noinputfiles = FALSE;
 		do_string(*argv);
+		interactive = save_state;
+
+	    } else if (!strncmp(*argv, "-slow", 2) || !strcmp(*argv, "--slow")) {
+		slow_font_startup = TRUE;
 
 	    } else if (!strncmp(*argv, "-d", 2) || !strcmp(*argv, "--default-settings")) {
 		/* Ignore this; it already had its effect */
@@ -638,10 +656,11 @@ RECOVER_FROM_ERROR_IN_DASH:
 		    fprintf(stderr, "syntax:  gnuplot -c scriptname args\n");
 		    gp_exit(EXIT_FAILURE);
 		}
-		for (i=0; i<argc; i++)
+		call_argc = GPMIN(9, argc - 1);
+		for (i = 0; i < call_argc; i++) {
 		    /* Need to stash argv[i] somewhere visible to load_file() */
-		    call_args[i] = gp_strdup(argv[i+1]);
-		call_argc = argc - 1;
+		    call_args[i] = gp_strdup(argv[i + 1]);
+		}
 
 		load_file(loadpath_fopen(*argv, "r"), gp_strdup(*argv), 5);
 		gp_exit(EXIT_SUCCESS);
@@ -656,11 +675,12 @@ RECOVER_FROM_ERROR_IN_DASH:
     }
 
     /* take commands from stdin */
-    if (noinputfiles)
+    if (noinputfiles) {
 	while (!com_line())
 	    ctrlc_flag = FALSE; /* reset asynchronous Ctrl-C flag */
+    }
 
-#ifdef _Windows
+#ifdef _WIN32
     /* On Windows, handle 'persist' by keeping the main input loop running (windows/wxt), */
     /* but only if there are any windows open. Note that qt handles this properly. */
     if (persist_cl) {
@@ -695,7 +715,7 @@ RECOVER_FROM_ERROR_IN_DASH:
 
     /* HBB 20040223: Not all compilers like exit() to end main() */
     /* exit(exit_status); */
-#if ! defined(_Windows)
+#if ! defined(_WIN32)
     /* Windows does the cleanup later */
     gp_exit_cleanup();
 #endif
@@ -725,7 +745,8 @@ init_constants()
     (void) Gcomplex(&udv_pi.udv_value, M_PI, 0.0);
     udv_NaN = get_udv_by_name("NaN");
     (void) Gcomplex(&(udv_NaN->udv_value), not_a_number(), 0.0);
-    udv_NaN->udv_undef = FALSE;
+    udv_I = get_udv_by_name("I");
+    (void) Gcomplex(&(udv_I->udv_value), 0.0, 1.0);
 }
 
 /*
@@ -741,27 +762,38 @@ init_session()
 	/* Undefine any previously-used variables */
 	del_udv_by_name("",TRUE);
 
-	/* Restore default colors before loadin local preferences */
+	/* Clear any previous customization of linetypes */
+	while (first_perm_linestyle != NULL)
+	    delete_linestyle(&first_perm_linestyle, NULL, first_perm_linestyle);
+
+	/* Restore default colors before loading local preferences */
 	set_colorsequence(1);
 
+	/* Reset program variables not handled by 'reset' */
+	overflow_handling = INT64_OVERFLOW_TO_FLOAT;
+	suppress_warnings = FALSE;
+
+	/* Reset voxel data structures if supported */
+	init_voxelsupport();
+
 	/* Make sure all variables start in the same state 'reset'
 	 * would set them to.
 	 */
 	reset_command();	/* FIXME: this does c_token++ */
 	load_rcfile(0);		/* System-wide gnuplotrc if configured */
-	load_rcfile(1);		/* ./.gnuplot if configured */
 
 	/* After this point we allow pipes and system commands */
 	successful_initialization = TRUE;
 
 	load_rcfile(2);		/* ~/.gnuplot */
+	load_rcfile(3);		/* ~/.config/gnuplot/gnuplotrc */
 }
 
 /*
  * Read commands from an initialization file.
  * where = 0: look for gnuplotrc in system shared directory
- * where = 1: look for .gnuplot in current directory
- * where = 2: look for .gnuplot in home directory
+ * where = 1: look for .gnuplot in current directory (DEPRECATED)
+ * where = 2: look for .gnuplot in user's home directory
  */
 static void
 load_rcfile(int where)
@@ -774,29 +806,16 @@ load_rcfile(int where)
 
     if (where == 0) {
 #ifdef GNUPLOT_SHARE_DIR
-# if defined(_Windows)
-	/* retrieve path relative to gnuplot executable,
-	 * whose path is in szModuleName (winmain.c) */
-	rcfile = gp_alloc(strlen((char *)szPackageDir) + 1
-	       + strlen(GNUPLOT_SHARE_DIR) + 1 + strlen("gnuplotrc") + 1, "rcfile");
-	strcpy(rcfile, (char *)szPackageDir);
-	PATH_CONCAT(rcfile, GNUPLOT_SHARE_DIR);
+# if defined(_WIN32) || defined(MSDOS) || defined(OS2)
+	rcfile = RelativePathToGnuplot(GNUPLOT_SHARE_DIR "\\gnuplotrc");
 # else
 	rcfile = (char *) gp_alloc(strlen(GNUPLOT_SHARE_DIR) + 1 + strlen("gnuplotrc") + 1, "rcfile");
 	strcpy(rcfile, GNUPLOT_SHARE_DIR);
-# endif
 	PATH_CONCAT(rcfile, "gnuplotrc");
+# endif
 	plotrc = fopen(rcfile, "r");
 #endif
 
-    } else if (where == 1) {
-#ifdef USE_CWDRC
-    /* Allow check for a .gnuplot init file in the current directory */
-    /* This is a security risk, as someone might leave a malicious   */
-    /* init file in a shared directory.                              */
-	plotrc = fopen(PLOTRC, "r");
-#endif /* !USE_CWDRC */
-
     } else if (where == 2 && user_homedir) {
 	/* length of homedir + directory separator + length of file name + \0 */
 	int len = (user_homedir ? strlen(user_homedir) : 0) + 1 + strlen(PLOTRC) + 1;
@@ -804,6 +823,13 @@ load_rcfile(int where)
 	strcpy(rcfile, user_homedir);
 	PATH_CONCAT(rcfile, PLOTRC);
 	plotrc = fopen(rcfile, "r");
+    } else if (where == 3) {
+#ifdef USE_XDG_BASEDIR
+	rcfile = xdg_get_path(kXDGConfigHome, "gnuplotrc",
+			TRUE /* use subdir */, FALSE /* not create */);
+	if (rcfile)
+	    plotrc = fopen(rcfile, "r");
+#endif
     }
 
     if (plotrc) {
@@ -815,6 +841,7 @@ load_rcfile(int where)
     free(rcfile);
 }
 
+
 void
 get_user_env()
 {
@@ -822,24 +849,21 @@ get_user_env()
 	const char *env_home;
 
 	if ((env_home = getenv(HOME))
-#ifdef WIN32
+#ifdef _WIN32
 	    || (env_home = appdata_directory())
 	    || (env_home = getenv("USERPROFILE"))
 #endif
-#ifndef VMS
 	    || (env_home = getenv("HOME"))
-#endif
 	   )
 	    user_homedir = (const char *) gp_strdup(env_home);
 	else if (interactive)
 	    int_warn(NO_CARET, "no HOME found");
     }
-    /* Hhm ... what about VMS? */
     if (user_shell == NULL) {
 	const char *env_shell;
 
 	if ((env_shell = getenv("SHELL")) == NULL)
-#if defined(MSDOS) || defined(_Windows) || defined(OS2)
+#if defined(MSDOS) || defined(_WIN32) || defined(OS2)
 	    if ((env_shell = getenv("COMSPEC")) == NULL)
 #endif
 		env_shell = SHELL;
@@ -897,33 +921,10 @@ ExecuteMacro(char *argv, int namelength)
 
     if (namelength >= sizeof(pszName))
 	return 1;
-    /* FIXME HBB 20010121: 3rd argument doesn't make sense. Either
-     * this should be sizeof(pszName), or it shouldn't use
-     * safe_strncpy(), here */
-    safe_strncpy(pszName, argv, namelength + 1);
+    safe_strncpy(pszName, argv, sizeof(pszName));
     rxArgStr = &argv[namelength];
     RXSTRPTR(rxRc) = NULL;
 
-#if 0
-    /*
-       C-like calling of function: program name is first
-       parameter.
-       In REXX you would have to use
-       Parse Arg param0, param1
-       to get the program name in param0 and the arguments in param1.
-
-       Some versions before gnuplot 3.7pl1 used a similar approach but
-       passed program name and arguments in a single string:
-       (==> Parse Arg param0 param1)
-     */
-
-    MAKERXSTRING(rxArg[0], pszName, strlen(pszName));
-    rxArgCount++;
-    if (*rxArgStr) {
-	MAKERXSTRING(rxArg[1], rxArgStr, strlen(rxArgStr));
-	rxArgCount++;
-    }
-#else
     /*
        REXX standard calling (gnuplot 3.7pl1 and above):
        The program name is not supplied and so all actual arguments
@@ -936,7 +937,6 @@ ExecuteMacro(char *argv, int namelength)
 	MAKERXSTRING(rxArg[0], rxArgStr, strlen(rxArgStr));
 	rxArgCount++;
     }
-#endif
 
     CallFromRexx = TRUE;
     rc = RexxStart(
@@ -992,10 +992,7 @@ RexxInterface(PRXSTRING rxCmd, PUSHORT pusErr, PRXSTRING rxRc)
 	/* Set variable gp_input_line.
 	   Watch out for line length of NOT_ZERO_TERMINATED strings ! */
 	cmdlen = rxCmd->strlength + 1;
-	/* FIXME HBB 20010121: 3rd argument doesn't make sense. Either
-	 * this should be gp_input_line_len, or it shouldn't use
-	 * safe_strncpy(), here */
-	safe_strncpy(gp_input_line, rxCmd->strptr, cmdlen);
+	safe_strncpy(gp_input_line, rxCmd->strptr, gp_input_line_len);
 	gp_input_line[cmdlen] = NUL;
 	rc = do_line();
 	*pusErr = RXSUBCOM_OK;
@@ -1059,7 +1056,7 @@ wrapper_for_write_history()
 	return;
     remove(expanded_history_filename);
     if (gnuplot_history_size < 0)
-    	write_history(expanded_history_filename);
+	write_history(expanded_history_filename);
     else
 	write_history_n(gnuplot_history_size, expanded_history_filename, "w");
 }
@@ -1073,3 +1070,23 @@ restrict_popen()
     if (!successful_initialization)
 	int_error(NO_CARET,"Pipes and shell commands not permitted during initialization");
 }
+
+
+#if defined(MSDOS) || defined(OS2)
+/* retrieve path relative to gnuplot executable */
+char *
+RelativePathToGnuplot(const char * path)
+{
+    char * rel_path, *s;
+
+    rel_path = (char * ) gp_alloc(strlen(progpath) + strlen(path) + 1, "RelativePathToGnuplot");
+    strcpy(rel_path, progpath);
+    /* progpath is guaranteed to have a trailing slash */
+    strcat(rel_path, path);
+    /* convert slashes to backslashes */
+    for (s = rel_path;*s != NUL; s++)
+	if (*s == DIRSEP2)
+	    *s = DIRSEP1;
+    return rel_path;
+}
+#endif
