@@ -243,7 +243,7 @@ sp_free(struct surface_points *sp)
  * iso_alloc() allocates a iso_curve structure that can hold 'num'
  * points.
  */
-struct iso_curve *
+static struct iso_curve *
 iso_alloc(int num)
 {
     struct iso_curve *ip;
@@ -263,7 +263,7 @@ iso_alloc(int num)
  * iso_extend() reallocates a iso_curve structure to hold "num"
  * points. This will either expand or shrink the storage.
  */
-void
+static void
 iso_extend(struct iso_curve *ip, int num)
 {
     if (num == ip->p_max)
@@ -286,7 +286,7 @@ iso_extend(struct iso_curve *ip, int num)
  * iso_free() releases any memory which was previously malloc()'d to hold
  *   iso curve points.
  */
-void
+static void
 iso_free(struct iso_curve *ip)
 {
     if (ip)
@@ -1156,7 +1156,7 @@ get_3ddata(struct surface_points *this_plot)
 		if (j < varcol)
 		    int_error(NO_CARET, "Not enough input columns");
 		if (j == varcol)
-		    color = this_plot->fill_properties.border_color.lt;
+		    color = z;
 		else
 		    color = v[varcol];
 		color_from_column(TRUE);
@@ -1176,6 +1176,14 @@ get_3ddata(struct surface_points *this_plot)
 		    color_from_column(FALSE);
 		}
 
+	    } else if (this_plot->plot_style == FILLEDCURVES) {
+		if (j != 3)
+		    int_error(NO_CARET, "this plot style wants 3 input columns x:y:z");
+		zlow = 0.0;
+		zhigh = v[2];
+		color_from_column(FALSE);
+		track_pm3d_quadrangles = TRUE;
+
 	    } else if (this_plot->plot_style == ZERRORFILL) {
 		if (j == 4) {
 		    zlow = v[2] - v[3];
@@ -1244,6 +1252,15 @@ get_3ddata(struct surface_points *this_plot)
 			color = rgb_from_colorspec( &lptmp.pm3d_color );
 			color_from_column(TRUE);
 		    }
+		    /* This allows "with polygons lc rgb variable" to set fillcolor.
+		     * FIXME: I'd rather that "lc" affected border color.
+		     */
+		    if (this_plot->lp_properties.pm3d_color.type == TC_RGB
+		    &&  this_plot->lp_properties.pm3d_color.value < 0.0) {
+			color = v[3];
+			color_from_column(TRUE);
+		    }
+		    /* This handles "with polygons fc rgb variable" */
 		    if (this_plot->fill_properties.border_color.type == TC_RGB
 		    &&  this_plot->fill_properties.border_color.value < 0) {
 			color = v[3];
@@ -1304,7 +1321,8 @@ get_3ddata(struct surface_points *this_plot)
 				this_plot->noautoscale,
 				cp->z=0;goto come_here_if_undefined);
 
-		if (this_plot->plot_style == ZERRORFILL) {
+		if ((this_plot->plot_style == ZERRORFILL)
+		||  (this_plot->plot_style == FILLEDCURVES)) {
 		    STORE_AND_UPDATE_RANGE(cp->CRD_ZLOW, zlow, cp->type, z_axis,
 				this_plot->noautoscale, goto come_here_if_undefined);
 		    STORE_AND_UPDATE_RANGE(cp->CRD_ZHIGH, zhigh, cp->type, z_axis,
@@ -2250,11 +2268,14 @@ eval_3dplots()
 	     *        ZERRORFILL is weird.
 	     */
 	    if ((this_plot->plot_style & PLOT_STYLE_HAS_FILL) && set_fillcolor) {
-		if (this_plot->plot_style == ZERRORFILL) {
+		if (this_plot->plot_style == ZERRORFILL
+		||  this_plot->plot_style == FILLEDCURVES) { 
 		    this_plot->fill_properties.border_color = this_plot->lp_properties.pm3d_color;
 		    this_plot->lp_properties.pm3d_color = fillcolor;
 		} else if (this_plot->plot_style == BOXES) {
 		    this_plot->lp_properties.pm3d_color = fillcolor;
+		} else if (this_plot->plot_style == CIRCLES) {
+		    this_plot->lp_properties.pm3d_color = fillcolor;
 		} else {
 		    this_plot->fill_properties.border_color = fillcolor;
 		}
@@ -2770,6 +2791,19 @@ eval_3dplots()
 	}
     }
 
+    /* The inside_zoom flag is set for both zoom and pan mouse operations.
+     * In either case apply_zoom() has loaded the requested plot limits
+     * into (axis)-> set_min and set_max.
+     */
+    if (inside_zoom) {
+	axis_array[FIRST_X_AXIS].min = axis_array[FIRST_X_AXIS].set_min;
+	axis_array[FIRST_X_AXIS].max = axis_array[FIRST_X_AXIS].set_max;
+	axis_array[FIRST_Y_AXIS].min = axis_array[FIRST_Y_AXIS].set_min;
+	axis_array[FIRST_Y_AXIS].max = axis_array[FIRST_Y_AXIS].set_max;
+	axis_array[FIRST_Z_AXIS].min = axis_array[FIRST_Z_AXIS].set_min;
+	axis_array[FIRST_Z_AXIS].max = axis_array[FIRST_Z_AXIS].set_max;
+    }
+
     if (nonlinear(&axis_array[FIRST_X_AXIS])) {
 	/* Transfer observed data or function ranges back to primary axes */
 	update_primary_axis_range(&axis_array[FIRST_X_AXIS]);
@@ -2804,6 +2838,18 @@ eval_3dplots()
 	setup_tics(&axis_array[SECOND_Y_AXIS], 20);
     }
 
+    /* Sanity check for auto-generated tics on logscale axes.
+     * If fewer than 3 tics were generated we will switch to using linear
+     * increment tic intervals when they are drawn by draw_3d_graphbox().
+     */
+    sanity_check_log_tics(FIRST_X_AXIS);
+    sanity_check_log_tics(FIRST_Y_AXIS);
+    sanity_check_log_tics(FIRST_Z_AXIS);
+    if (splot_map) {
+	sanity_check_log_tics(SECOND_X_AXIS);
+	sanity_check_log_tics(SECOND_Y_AXIS);
+    }
+
     set_plot_with_palette(plot_num, MODE_SPLOT);
     if (is_plot_with_palette()) {
 	set_cbminmax();
