@@ -89,7 +89,7 @@ static void pm3d_rearrange_part(struct iso_curve *, const int, struct iso_curve
 static int apply_lighting_model(struct coordinate *, struct coordinate *,
 				struct coordinate *, struct coordinate *,
 				double gray, TBOOLEAN gray_is_rgb );
-static void illuminate_one_quadrangle( quadrangle *q );
+static void illuminate_one_quadrangle(quadrangle *q, double gray, TBOOLEAN gray_is_rgb);
 
 static void filled_polygon(struct surface_points *from_plot, int index,
 			    gpdPoint *corners, int nv);
@@ -513,10 +513,6 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     if (this_plot->lp_properties.colormap)
 	private_colormap = this_plot->lp_properties.colormap;
 
-    /* Feb 2023:  not necessary and may be counter-productive
-	term_apply_lp_properties(&this_plot->lp_properties);
-     */
-
     if (at_which_z != PM3D_AT_BASE && at_which_z != PM3D_AT_TOP && at_which_z != PM3D_AT_SURFACE)
 	return;
     else
@@ -1132,6 +1128,30 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	term->layer(TERM_LAYER_END_PM3D_MAP);
 }				/* end of pm3d splotting mode */
 
+/*
+ * Adapt pm3d color assignment code to apply to triangles
+ * (used by splot with polygons)
+ */
+double
+pm3d_assign_triangle_z( double cb1, double cb2, double cb3 )
+{
+    double netz;
+    switch (pm3d.which_corner_color) {
+	default:
+	case PM3D_WHICHCORNER_MEDIAN:
+	case PM3D_WHICHCORNER_GEOMEAN:
+	case PM3D_WHICHCORNER_HARMEAN:
+	case PM3D_WHICHCORNER_MEAN: netz = (cb1 + cb2 + cb3) / 3.; break;
+	case PM3D_WHICHCORNER_C1: netz = cb1; break;
+	case PM3D_WHICHCORNER_C2: netz = cb2; break;
+	case PM3D_WHICHCORNER_C3: netz = cb3; break;
+	case PM3D_WHICHCORNER_C4: netz = cb1; break;
+	case PM3D_WHICHCORNER_MIN: netz = minimum4(cb1, cb2, cb3, cb1); break;
+	case PM3D_WHICHCORNER_MAX: netz = maximum4(cb1, cb2, cb3, cb1); break;
+	case PM3D_WHICHCORNER_RMS: netz = sqrt( (cb1*cb1 + cb2*cb2 + cb3*cb3)/3. ); break;
+    }
+    return netz;
+}
 
 /*
  * unset pm3d for the reset command
@@ -1186,7 +1206,7 @@ pm3d_draw_one(struct surface_points *plot)
 
 /*
  * Add one pm3d quadrangle to the mix.
- * Called by zerrorfill() and by plot3d_boxes().
+ * Called by zerrorfill() plot3d_boxes() do_polygon() plot3d_polygons().
  * Also called by vplot_isosurface().
  */
 void
@@ -1252,21 +1272,30 @@ pm3d_add_polygon(struct surface_points *plot, gpdPoint corners[], int vertices)
     } else if (plot->pm3d_color_from_column
 		&& !(plot->plot_style == POLYGONS)) {
 	/* This is the usual path for 'splot with boxes' */
-	color_from_rgbvar = TRUE;
 	if (pm3d_shade.strength > 0) {
-	    q->gray = plot->lp_properties.pm3d_color.lt;
-	    illuminate_one_quadrangle(q);
+	    color_from_rgbvar = TRUE;
+	    illuminate_one_quadrangle(q, plot->lp_properties.pm3d_color.lt, color_from_rgbvar);
 	} else {
 	    q->qcolor = plot->lp_properties.pm3d_color.lt;
 	    q->gray = PM3D_USE_RGB_COLOR_INSTEAD_OF_GRAY;
 	}
 
     } else if (plot->lp_properties.pm3d_color.type == TC_Z) {
-	/* This is a special case for 'splot with boxes lc palette z' */
-	q->gray = cb2gray(corners[1].z);
-	color_from_rgbvar = FALSE;
+	if (plot->plot_style == POLYGONS) {
+	    if (pm3d_shade.strength > 0) {
+		q->gray = corners[0].c;
+		color_from_rgbvar = TRUE;
+	    } else {
+		q->qcolor = corners[0].c;
+		q->gray = PM3D_USE_RGB_COLOR_INSTEAD_OF_GRAY;
+	    }
+	} else {
+	    /* This is a special case for 'splot with boxes lc palette z' */
+	    q->gray = cb2gray(corners[1].z);
+	    color_from_rgbvar = FALSE;
+	}
 	if (pm3d_shade.strength > 0)
-	    illuminate_one_quadrangle(q);
+	    illuminate_one_quadrangle(q, q->gray, color_from_rgbvar);
 
     } else if (plot->plot_style == ISOSURFACE
            ||  plot->plot_style == POLYGONS) {
@@ -1294,9 +1323,8 @@ pm3d_add_polygon(struct surface_points *plot, gpdPoint corners[], int vertices)
 	}
 	q->qcolor = rgb_color;
 	if (pm3d_shade.strength > 0) {
-	    q->gray = rgb_color;
 	    color_from_rgbvar = TRUE;
-	    illuminate_one_quadrangle(q);
+	    illuminate_one_quadrangle(q, rgb_color, color_from_rgbvar);
 	}
 
     } else {
@@ -1446,7 +1474,7 @@ pm3d_init_lighting_model()
  * Layer on layer of coordinate conventions
  */
 static void
-illuminate_one_quadrangle( quadrangle *q )
+illuminate_one_quadrangle( quadrangle *q, double gray, TBOOLEAN gray_is_rgb )
 {
     struct coordinate c1, c2, c3, c4;
     vertex vtmp;
@@ -1459,7 +1487,7 @@ illuminate_one_quadrangle( quadrangle *q )
     c3.x = vtmp.x; c3.y = vtmp.y; c3.z = vtmp.z;
     map3d_xyz(q->vertex.corners[3].x, q->vertex.corners[3].y, q->vertex.corners[3].z, &vtmp);
     c4.x = vtmp.x; c4.y = vtmp.y; c4.z = vtmp.z;
-    q->gray = apply_lighting_model( &c1, &c2, &c3, &c4, q->gray, color_from_rgbvar);
+    q->gray = apply_lighting_model( &c1, &c2, &c3, &c4, gray, gray_is_rgb);
 }
 
 /*
@@ -1779,6 +1807,8 @@ clip_filled_polygon( struct surface_points *from_plot, int index,
 	    zmin = from_plot->zclip[index].zlow;
 	if (zmax > from_plot->zclip[index].zhigh)
 	    zmax = from_plot->zclip[index].zhigh;
+	if (zmin > zmax)
+	    return -1;
     }
 
     for (current = 0; current < nv; current++) {
@@ -2013,11 +2043,11 @@ split_intersecting_surface_tiles()
     quadrangle *q1, *q2, *qnew;
     int iq1, iq2;
     int s0, s1, s2, s3;		/* -1 = bottom surface; 0 = intersection; 1 = top surface */
-    int nv1, nv2;		/* number of vertices in piece1, piece2 */
+    int nv1, nv2, nv3;		/* number of vertices in piece1, piece2, piece3 */
     gpdPoint *ptlist[8];	/* list of 4 vertices of q1 plus possible intersections */
     int side[8];		/* top/bottom/intersection status of points in ptlist */
-    gpdPoint intersection[2];	/* We know there are only two intersections */
-    gpdPoint piece1[5], piece2[5];
+    gpdPoint intersection[4];	/* usually 0 or 2, pathological case from bent tile gives 4 */
+    gpdPoint piece1[8], piece2[8], piece3[8];
     int pass;
 
     /* These calculations all assume a simple grid shared by all plots in the superposition */
@@ -2063,15 +2093,15 @@ split_intersecting_surface_tiles()
 	    /* Found a pair of intersecting quadrangles!
 	     * Make sure the list is long enough to hold new pieces.
 	     */
-	    if (reserve_quadrangles(4, 4*samples_1)) {
+	    if (reserve_quadrangles(6, 4*samples_1)) {
 		q1 = &(quadrangles[iq1]);
 		q2 = &(quadrangles[iq2]);
 	    }
 
 	    /* Construct a list of pointers to the 4 original vertices of q1
-	     * and to the 2 intersection points.  We don't yet know which sides
-	     * the intersections lie on, so we leave room for 2 NULL pointers
-	     * corresponding to the sides with no intersection.
+	     * and up to 4 intersection points.  We don't yet know which sides
+	     * contain intersections, so we initialize them to NULL and replace
+	     * with actual intersections as we go.
 	     */
 	    ptlist[1] = ptlist[3] = ptlist[5] = ptlist[7] = NULL;
 	    side[1] = side[3] = side[5] = side[7] = 0;
@@ -2114,19 +2144,59 @@ split_intersecting_surface_tiles()
 				      &intersection[next_int]);
 		    ptlist[7] = &intersection[next_int++];
 		}
-		/* Should never happen */
-		if (next_int != 2)
-		    int_error(NO_CARET, "calculation of surface intersection failed n=%d", next_int);
 
-		/* Copy the "top" vertices to piece1, "bottom" to piece2 */
-		nv1 = nv2 = 0;
-		for (i=0; i<8; i++) {
-		    if (ptlist[i] != NULL && side[i] >= 0)
-			piece1[nv1++] = *ptlist[i];
-		    if (ptlist[i] != NULL && side[i] <= 0)
-			piece2[nv2++] = *ptlist[i];
+		/* Normal case: 2 intersections
+		 * Copy the "top" vertices to piece1, "bottom" to piece2
+		 */
+		if (next_int == 2) {
+		    nv1 = nv2 = nv3 = 0;
+		    for (i=0; i<8; i++) {
+			if (ptlist[i] != NULL && side[i] >= 0)
+			    piece1[nv1++] = *ptlist[i];
+			if (ptlist[i] != NULL && side[i] <= 0)
+			    piece2[nv2++] = *ptlist[i];
+		    }
 		}
 
+		/* Pathological case of a bent quadrangle, two diagonal corners on one side
+		 * and two diagonal corners on the other.  Each quadrangle will be replaced
+		 * by two disconnected triangles and a hexagon.
+		 */
+		else if (next_int == 4) {
+		    nv1 = nv2 = nv3 = 0;
+		    if (side[0] > 0) {
+			piece1[nv1++] = *ptlist[0];
+			piece1[nv1++] = *ptlist[1];
+			piece1[nv1++] = *ptlist[7];
+			piece2[nv2++] = *ptlist[1];
+			piece2[nv2++] = *ptlist[2];
+			piece2[nv2++] = *ptlist[3];
+			piece2[nv2++] = *ptlist[5];
+			piece2[nv2++] = *ptlist[6];
+			piece2[nv2++] = *ptlist[7];
+			piece3[nv3++] = *ptlist[3];
+			piece3[nv3++] = *ptlist[4];
+			piece3[nv3++] = *ptlist[5];
+		    } else {
+			piece1[nv1++] = *ptlist[0];
+			piece1[nv1++] = *ptlist[1];
+			piece1[nv1++] = *ptlist[3];
+			piece1[nv2++] = *ptlist[4];
+			piece1[nv2++] = *ptlist[5];
+			piece1[nv2++] = *ptlist[7];
+			piece2[nv2++] = *ptlist[2];
+			piece2[nv2++] = *ptlist[3];
+			piece2[nv2++] = *ptlist[4];
+			piece3[nv3++] = *ptlist[5];
+			piece3[nv3++] = *ptlist[6];
+			piece3[nv3++] = *ptlist[7];
+		    }
+		}
+
+		/* Should never happen */
+		else
+		    int_error(NO_CARET, "calculation of surface intersection failed n=%d", next_int);
+
 		/* Add piece1 as a new quadrangle at the end of the list */
 		if (nv1 > 2) {
 		    qnew = &quadrangles[current_quadrangle++];
@@ -2137,7 +2207,6 @@ split_intersecting_surface_tiles()
 #ifdef WITH_2ND_SORTKEY
 		    qnew->sequence = qt->sequence;
 #endif
-
 		    qnew->vertex.corners[0] = piece1[0];
 		    qnew->vertex.corners[1] = piece1[1];
 		    qnew->vertex.corners[2] = piece1[2];
@@ -2147,7 +2216,7 @@ split_intersecting_surface_tiles()
 			qnew->vertex.corners[3] = piece1[2];
 			qnew->type = QUAD_TYPE_TRIANGLE;
 		    }
-		    if (nv1 == 5) {
+		    if (nv1 > 4) {
 			gpdPoint *save_corners = get_polygon(nv1);
 			qnew->vertex.array_index = current_polygon;
 			memcpy(save_corners, piece1, nv1 * sizeof(gpdPoint));
@@ -2166,7 +2235,6 @@ split_intersecting_surface_tiles()
 #ifdef WITH_2ND_SORTKEY
 		    qnew->sequence = qt->sequence;
 #endif
-
 		    qnew->vertex.corners[0] = piece2[0];
 		    qnew->vertex.corners[1] = piece2[1];
 		    qnew->vertex.corners[2] = piece2[2];
@@ -2176,7 +2244,7 @@ split_intersecting_surface_tiles()
 			qnew->vertex.corners[3] = piece2[2];
 			qnew->type = QUAD_TYPE_TRIANGLE;
 		    }
-		    if (nv2 == 5) {
+		    if (nv2 > 4) {
 			gpdPoint *save_corners = get_polygon(nv2);
 			qnew->vertex.array_index = current_polygon;
 			qnew->type = QUAD_TYPE_LARGEPOLYGON;
@@ -2184,6 +2252,24 @@ split_intersecting_surface_tiles()
 			save_corners[2].c = nv2;
 		    }
 		}
+
+		/* Add piece3 as a new triangle at the end of the list */
+		if (nv3 > 2) {
+		    qnew = &quadrangles[current_quadrangle++];
+		    qnew->from_plot = qt->from_plot;
+		    qnew->gray = qt->gray;
+		    qnew->index = qt->index;
+		    qnew->qcolor = qt->qcolor;
+#ifdef WITH_2ND_SORTKEY
+		    qnew->sequence = qt->sequence;
+#endif
+		    qnew->vertex.corners[0] = piece3[0];
+		    qnew->vertex.corners[1] = piece3[1];
+		    qnew->vertex.corners[2] = piece3[2];
+		    qnew->vertex.corners[3] = piece3[2];
+		    qnew->type = QUAD_TYPE_TRIANGLE;
+		}
+
 	    }
 
 	    /* Mark original q1 and q2 as not to be drawn */
