@@ -382,6 +382,9 @@ void qt_connectToServer()
 
 bool qt_processTermEvent(gp_event_t* event)
 {
+	// remember initial state (the original gets cleared by do_event())
+	TBOOLEAN was_paused_for_mouse = paused_for_mouse;
+
 	// Intercepts resize event
 	if (event->type == GE_fontprops)
 	{
@@ -435,9 +438,8 @@ bool qt_processTermEvent(gp_event_t* event)
 		paused_for_mouse = 0;
 		return true;
 	}
-	if (event->type == GE_reset)
+	if (event->type == GE_reset && was_paused_for_mouse)
 	{
-		paused_for_mouse = 0;
 		return true;
 	}
 
@@ -636,10 +638,22 @@ void qt_text_wrapper()
 
 	for (int i = 0; i < 4; i++)
 	{
-		qt->out << (axis_array[axis_order[i]].ticmode != NO_TICS); // Axis active or not
-		qt->out << axis_array[axis_order[i]].min;
-		double lower = double(axis_array[axis_order[i]].term_lower);
-		double scale = double(axis_array[axis_order[i]].term_scale);
+		struct axis *mousing_axis = &axis_array[axis_order[i]];
+		double log_base;
+
+		qt->out << (mousing_axis->ticmode != NO_TICS); // Axis active or not
+
+		if (mousing_axis->log)
+			log_base = mousing_axis->log_base;
+		else if (nonlinear(mousing_axis))
+			log_base = -1;
+		else
+			log_base = 0;
+		if (mousing_axis->log && mousing_axis->linked_to_primary)
+			mousing_axis = mousing_axis->linked_to_primary;
+		qt->out << mousing_axis->min;
+		double lower = mousing_axis->term_lower;
+		double scale = mousing_axis->term_scale;
 		// Reverse the y axis
 		if (i % 2)
 		{
@@ -647,7 +661,7 @@ void qt_text_wrapper()
 			scale *= -1;
 		}
 		qt->out << lower/qt_oversamplingF << scale/qt_oversamplingF;
-		qt->out << (axis_array[axis_order[i]].log ? axis_array[axis_order[i]].log_base : 0.);
+		qt->out << log_base;
 	}
 	// Flag whether this was a 3D plot (not mousable in 'persist' mode)
 	qt->out << qt_is_3Dplot;
@@ -1043,7 +1057,7 @@ int qt_waitforinput(int options)
 		if (options == TERM_ONLY_CHECK_MOUSING)
 			return '\0';
 		else
-			return getchar();
+			read_and_return_character();
 	}
 
 	// Gnuplot event loop
@@ -1133,7 +1147,7 @@ int qt_waitforinput(int options)
 	if (options == TERM_ONLY_CHECK_MOUSING)
 		return '\0';
 
-	return getchar();
+	read_and_return_character();
 
 #else // _WIN32, Windows console and wgnuplot
 #ifdef WGP_CONSOLE
