@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: tabulate.c,v 1.22.2.1 2015/01/16 18:47:57 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - tabulate.c */
 
 /*[
@@ -62,8 +58,11 @@ static char *RCSid() { return RCSid("$Id: tabulate.c,v 1.22.2.1 2015/01/16 18:47
 FILE *table_outfile = NULL;
 udvt_entry *table_var = NULL;
 TBOOLEAN table_mode = FALSE;
+char *table_sep = NULL;
+struct at_type *table_filter_at = NULL;
 
-static char *expand_newline __PROTO((const char *in));
+static char *expand_newline(const char *in);
+static TBOOLEAN blanks_needed(curve_points *this_plot);
 
 static FILE *outfile;
 
@@ -93,9 +92,6 @@ output_number(double coord, int axis, char *buffer) {
 	    gstrftime(buffer+1, BUFFERSIZE-1, axis_array[axis].formatstring, coord);
 	while (strchr(buffer,'\n')) {*(strchr(buffer,'\n')) = ' ';}
 	strcat(buffer,"\"");
-    } else if (axis_array[axis].log) {
-	double x = pow(axis_array[axis].base, coord);
-	gprintf(buffer, BUFFERSIZE, axis_array[axis].formatstring, 1.0, x);
     } else
 	gprintf(buffer, BUFFERSIZE, axis_array[axis].formatstring, 1.0, coord);
     strcat(buffer, " ");
@@ -175,8 +171,11 @@ print_table(struct curve_points *current_plot, int plot_num)
 	case VECTOR:
 	    len = strappend(&line, &size, len, " delta_x delta_y");
 	    break;
-	case LINES:
 	case POINTSTYLE:
+	    if (current_plot->plot_filter == FILTER_ZSORT)
+		len = strappend(&line, &size, len, " z");
+	    break;
+	case LINES:
 	case LINESPOINTS:
 	case DOTS:
 	case IMPULSES:
@@ -191,6 +190,10 @@ print_table(struct curve_points *current_plot, int plot_num)
 	case RGBA_IMAGE:
 	    len = strappend(&line, &size, len, "  red green blue alpha");
 	    break;
+	case POLYGONS:
+	    if (df_no_use_specs >= 3)
+		len = strappend(&line, &size, len, " z");
+	    break;
 
 	default:
 	    if (interactive)
@@ -200,6 +203,9 @@ print_table(struct curve_points *current_plot, int plot_num)
 	    break;
 	}
 
+	if (current_plot->plot_filter == FILTER_BINS)
+	    len = strappend(&line, &size, len, "  N");
+
 	if (current_plot->varcolor)
 	    len = strappend(&line, &size, len, "  color");
 
@@ -225,6 +231,9 @@ print_table(struct curve_points *current_plot, int plot_num)
 
 	} else {
 	    int plotstyle = current_plot->plot_style;
+	    int type;
+	    TBOOLEAN replace_undefined_with_blank = blanks_needed(current_plot);
+
 	    if (plotstyle == HISTOGRAMS && current_plot->histogram->type == HT_ERRORBARS)
 		plotstyle = YERRORBARS;
 
@@ -232,7 +241,12 @@ print_table(struct curve_points *current_plot, int plot_num)
 		i++, point++) {
 
 		/* Reproduce blank lines read from original input file, if any */
-		if (!memcmp(point, &blank_data_line, sizeof(struct coordinate))) {
+		/* NB: complicated test is necessary because if struct coordinate
+		 *     is padded, the padding has not been initialized and memcmp
+		 *     chokes on uninitialized bytes (or anyhow valgrind thinks so)
+		 */
+		if (point->type == UNDEFINED
+		&&  !memcmp(&point->x, &blank_data_line.x, 7*sizeof(coordval))) {
 		    print_line("");
 		    continue;
 		}
@@ -290,8 +304,11 @@ print_table(struct curve_points *current_plot, int plot_num)
 			OUTPUT_NUMBER((point->xhigh - point->x), current_plot->x_axis);
 			OUTPUT_NUMBER((point->yhigh - point->y), current_plot->y_axis);
 			break;
-		    case LINES:
 		    case POINTSTYLE:
+			if (current_plot->plot_filter == FILTER_ZSORT)
+			    OUTPUT_NUMBER(point->z, FIRST_Z_AXIS);
+			break;
+		    case LINES:
 		    case LINESPOINTS:
 		    case DOTS:
 		    case IMPULSES:
@@ -299,11 +316,21 @@ print_table(struct curve_points *current_plot, int plot_num)
 		    case FSTEPS:
 		    case HISTEPS:
 			break;
+		    case POLYGONS:
+			if (df_no_use_specs >= 3)
+			    OUTPUT_NUMBER(point->z, FIRST_Z_AXIS);
+			break;
 		    default:
 			/* ? */
 			break;
 		} /* switch(plot type) */
 
+		/* FIXME: which is in the right place - BINS? or ZSORT? */
+		if (current_plot->plot_filter == FILTER_BINS) {
+		    snprintf(buffer, BUFFERSIZE, " %4d", (int)point->z);
+		    len = strappend(&line, &size, len, buffer);
+		}
+
 		if (current_plot->varcolor) {
 		    double colorval = current_plot->varcolor[i];
 		    if ((current_plot->lp_properties.pm3d_color.value < 0.0)
@@ -317,12 +344,20 @@ print_table(struct curve_points *current_plot, int plot_num)
 		    }
 		}
 
+		type = current_plot->points[i].type;
+
 		snprintf(buffer, BUFFERSIZE, " %c",
-		    current_plot->points[i].type == INRANGE
-		    ? 'i' : current_plot->points[i].type == OUTRANGE
-		    ? 'o' : 'u');
+			type == INRANGE ? 'i' : type == OUTRANGE ? 'o' : 
+			type == EXCLUDEDRANGE ? 'e' : 'u');
 		strappend(&line, &size, len, buffer);
-		print_line(line);
+
+		/* cp_implode() inserts dummy undefined point between curves */
+		/* but datafiles use a blank line for this purpose */
+		if (type == UNDEFINED && replace_undefined_with_blank)
+		    print_line("");
+		else 
+		    print_line(line);
+
 	    } /* for(point i) */
 	}
 
@@ -391,14 +426,18 @@ print_3dtable(int pcount)
 	case IMPULSES:
 	case DOTS:
 	case VECTOR:
-	case IMAGE:
 	    break;
+	case IMAGE:
+	case RGBIMAGE:
+	case RGBA_IMAGE:
+	    break;
+
 	default:
 	    fprintf(stderr, "Tabular output of this 3D plot style not implemented\n");
 	    continue;
 	}
 
-	if (draw_surface) {
+	if (draw_surface && !this_plot->opt_out_of_surface) {
 	    struct iso_curve *icrvs;
 	    int curve;
 
@@ -412,11 +451,22 @@ print_3dtable(int pcount)
 			curve, icrvs->p_count);
 		print_line(line);
 		len = sprintf(line, "# x y z");
-		if (this_plot->plot_style == VECTOR) {
+		tail = NULL;  /* Just to shut up a compiler warning */
+
+		switch (this_plot->plot_style) {
+		case VECTOR:
 		    tail = icrvs->next->points;
 		    len = strappend(&line, &size, len, " delta_x delta_y delta_z");
-		} else {
-		    tail = NULL;  /* Just to shut up a compiler warning */
+		    break;
+		case IMAGE:
+		    len = strappend(&line, &size, len, "  pixel");
+		    break;
+		case RGBIMAGE:
+		case RGBA_IMAGE:
+		    len = strappend(&line, &size, len, "  red green blue alpha");
+		    break;
+		default:
+		    break;
 		}
 
 		strappend(&line, &size, len, " type");
@@ -438,6 +488,12 @@ print_3dtable(int pcount)
 		    } else if (this_plot->plot_style == IMAGE) {
 			snprintf(buffer, BUFFERSIZE, "%g ", point->CRD_COLOR);
 			len = strappend(&line, &size, len, buffer);
+		    } else if (this_plot->plot_style == RGBIMAGE
+			   ||  this_plot->plot_style == RGBA_IMAGE) {
+			snprintf(buffer, BUFFERSIZE, "%4d %4d %4d %4d ", 
+			        (int)point->CRD_R, (int)point->CRD_G,
+			        (int)point->CRD_B, (int)point->CRD_A);
+			len = strappend(&line, &size, len, buffer);
 		    }
 		    snprintf(buffer, BUFFERSIZE, "%c",
 			    point->type == INRANGE
@@ -445,12 +501,12 @@ print_3dtable(int pcount)
 			    ? 'o' : 'u');
 		    strappend(&line, &size, len, buffer);
 		    print_line(line);
-		} /* for(point) */
-	    } /* for(icrvs) */
+		} /* for (point) */
+	    } /* for (icrvs) */
 	    print_line("");
-	} /* if(draw_surface) */
+	} /* if (draw_surface) */
 
-	if (draw_contour) {
+	if (draw_contour && ! this_plot->opt_out_of_contours) {
 	    int number = 0;
 	    struct gnuplot_contours *c = this_plot->contours;
 
@@ -458,7 +514,7 @@ print_3dtable(int pcount)
 		int count = c->num_pts;
 		struct coordinate *point = c->coords;
 
-		if (c->isNewLevel)
+		if (c->isNewLevel) {
 		    /* don't display count - contour split across chunks */
 		    /* put # in case user wants to use it for a plot */
 		    /* double blank line to allow plot ... index ... */
@@ -466,6 +522,7 @@ print_3dtable(int pcount)
 		    snprintf(line, size, "# Contour %d, label: %s",
 			    number++, c->label);
 		    print_line(line);
+		}
 
 		for (; --count >= 0; ++point) {
 		    line[0] = NUL;
@@ -494,7 +551,7 @@ print_3dtable(int pcount)
 static char *
 expand_newline(const char *in)
 {
-    char *tmpstr = (char *) gp_alloc(2 * strlen(in), "enl");
+    char *tmpstr = (char *) gp_alloc(2 * strlen(in) + 1, "enl");
     const char *s = in;
     char *t = tmpstr;
     do {
@@ -506,3 +563,93 @@ expand_newline(const char *in)
     } while (*s++);
     return tmpstr;
 }
+
+/* Some plot styles depend on an internal convention that a data point
+ * with type UNDEFINED indicates the break between separate curves within
+ * a single plot.  The equivalent convention on input is a blank line.
+ * This routine tests for styles that should convert the UNDEFINED point
+ * to a blank line on output.
+ */
+static TBOOLEAN
+blanks_needed(curve_points *this_plot)
+{
+    switch (this_plot->plot_smooth) {
+	/* These smooth styles called cp_implode() */
+	case SMOOTH_UNIQUE:
+	case SMOOTH_FREQUENCY:
+	case SMOOTH_FREQUENCY_NORMALISED:
+	case SMOOTH_CUMULATIVE:
+	case SMOOTH_CUMULATIVE_NORMALISED:
+	case SMOOTH_CSPLINES:
+	case SMOOTH_ACSPLINES:
+	case SMOOTH_SBEZIER:
+	case SMOOTH_MONOTONE_CSPLINE:
+	    return TRUE;
+	/* These ones did not */
+	case SMOOTH_NONE:
+	case SMOOTH_BEZIER:
+	case SMOOTH_KDENSITY:
+	    break;
+	/* These smooth styles also use the UNDEFINED point convention */
+	case SMOOTH_PATH:
+	    return TRUE;
+	default:
+	    break;
+    }
+    return FALSE;
+}
+
+/*
+ * Called from plot2d.c (get_data) for "plot with table"
+ */
+TBOOLEAN
+tabulate_one_line(double v[], struct value str[], int ncols)
+{
+    int col;
+    FILE *outfile = (table_outfile) ? table_outfile : gpoutfile;
+    struct value keep;
+
+    if (table_filter_at) {
+	evaluate_inside_using = TRUE;
+	evaluate_at(table_filter_at, &keep);
+	evaluate_inside_using = FALSE;
+	if (undefined || isnan(real(&keep)) || real(&keep) == 0)
+	    return FALSE;
+    }
+
+    if (table_var == NULL) {
+	char sep = (table_sep && *table_sep) ? *table_sep : '\t';
+	for (col = 0; col < ncols; col++) {
+	    if (str[col].type == STRING)
+		fprintf(outfile, " %s", str[col].v.string_val);
+	    else
+		fprintf(outfile, " %g", v[col]);
+	    if (col < ncols-1)
+		fprintf(outfile, "%c", sep);
+	}
+	fprintf(outfile, "\n");
+    } else {
+	char buf[64]; /* buffer large enough to hold %g + 2 extra chars */
+	char sep = (table_sep && *table_sep) ? *table_sep : '\t';
+	size_t size = sizeof(buf);
+	char *line = (char *) gp_alloc(size, "");
+	size_t len = 0;
+
+	line[0] = NUL;
+	for (col = 0; col < ncols; col++) {
+	    if (str[col].type == STRING) {
+		len = strappend(&line, &size, 0, str[col].v.string_val);
+	    } else {
+		snprintf(buf, sizeof(buf), " %g", v[col]);
+		len = strappend(&line, &size, len, buf);
+	    }
+	    if (col < ncols-1) {
+		snprintf(buf, sizeof(buf), " %c", sep);
+		len = strappend(&line, &size, len, buf);
+	    }
+	}
+	append_to_datablock(&table_var->udv_value, line);
+    }
+ 
+    return TRUE;
+}
