@@ -1,7 +1,3 @@
-/*
- * $Id: gp_cairo.c,v 1.87.2.6 2015/11/06 19:26:13 sfeam Exp $
- */
-
 /* GNUPLOT - gp_cairo.c */
 
 /*[
@@ -122,6 +118,15 @@ static int avg_vchar = 150;
 static void gp_cairo_fill(plot_struct *plot, int fillstyle, int fillpar);
 static void gp_cairo_fill_pattern(plot_struct *plot, int fillstyle, int fillpar);
 
+/* Boxed text support */
+static int bounding_box[4];
+static double bounding_xmargin = 1.0;
+static double bounding_ymargin = 1.0;
+static double box_rotation = 0.0;
+static double box_origin_x;
+static double box_origin_y;
+static TBOOLEAN in_textbox = FALSE;
+
 /* array of colors
  * FIXME could be shared with all gnuplot terminals */
 static rgb_color gp_cairo_colorlist[12] = {
@@ -139,7 +144,7 @@ static rgb_color gp_cairo_colorlist[12] = {
 {0.5,0.5,0.5} /* grey */
 };
 
-/* correspondance between gnuplot linetypes and terminal colors */
+/* correspondence between gnuplot linetypes and terminal colors */
 void gp_cairo_set_background( rgb_color background )
 {
 	gp_cairo_colorlist[0] = background;
@@ -174,7 +179,7 @@ void gp_cairo_initialize_plot(plot_struct *plot)
 
 	plot->current_x = -1; plot->current_y = -1;
 
-	strncpy(plot->fontname, "", sizeof(plot->fontname));
+	safe_strncpy(plot->fontname, "", sizeof(plot->fontname));
 	plot->fontsize = 1.0;
 	plot->encoding = S_ENC_DEFAULT;
 
@@ -185,6 +190,8 @@ void gp_cairo_initialize_plot(plot_struct *plot)
 	plot->oversampling = TRUE;
 	plot->oversampling_scale = GP_CAIRO_SCALE;
 
+	plot->upsampling_rate = 1.0;
+
 	plot->linecap = BUTT;
 
 	plot->hinting = 100;
@@ -196,14 +203,9 @@ void gp_cairo_initialize_plot(plot_struct *plot)
 	plot->polygon_path_last = NULL;
 
 	plot->interrupt = FALSE;
-}
 
-#ifdef EAM_BOXED_TEXT
-/* Boxed text support */
-static int bounding_box[4];
-static double bounding_xmargin = 1.0;
-static double bounding_ymargin = 1.0;
-#endif
+	in_textbox = FALSE;
+}
 
 /* set the transformation matrix of the context, and other details */
 /* NOTE : depends on the setting of xscale and yscale */
@@ -228,6 +230,11 @@ void gp_cairo_initialize_context(plot_struct *plot)
 			0.5, 0.5);
 	cairo_set_matrix(plot->cr, &matrix);
 
+	gp_cairo_set_lineprops(plot);
+}
+
+void gp_cairo_set_lineprops(plot_struct *plot)
+{
 	/* Default is square caps, mitered joins */
 	if (plot->linecap == ROUNDED) {
 	    cairo_set_line_cap  (plot->cr, CAIRO_LINE_CAP_ROUND);
@@ -241,7 +248,6 @@ void gp_cairo_initialize_context(plot_struct *plot)
 	    cairo_set_line_join (plot->cr, CAIRO_LINE_JOIN_MITER);
 	    cairo_set_miter_limit(plot->cr, 3.8);
 	}
-
 }
 
 
@@ -301,12 +307,12 @@ void gp_cairo_set_justify(plot_struct *plot, JUSTIFY mode)
 }
 
 
-void gp_cairo_set_font(plot_struct *plot, const char *name, int fontsize)
+void gp_cairo_set_font(plot_struct *plot, const char *name, float fontsize)
 {
     char *c;
     char *fname;
 
-	FPRINTF((stderr,"set_font \"%s\" %d\n", name,fontsize));
+	FPRINTF((stderr,"set_font \"%s\" %f\n", name,fontsize));
 
 	/* Split out Bold and Italic attributes from font name */
 	fname = strdup(name);
@@ -330,11 +336,17 @@ void gp_cairo_set_font(plot_struct *plot, const char *name, int fontsize)
 	} else
 	    plot->fontstyle = PANGO_STYLE_NORMAL;
 
-	strncpy( plot->fontname, fname, sizeof(plot->fontname) );
+	safe_strncpy( plot->fontname, fname, sizeof(plot->fontname) );
 	plot->fontsize = fontsize;
 	free(fname);
 }
 
+/* work-around for "bold font gets stuck" bug */
+void gp_cairo_clear_bold_font(plot_struct *plot)
+{
+	plot->fontweight = PANGO_WEIGHT_NORMAL;
+	plot->fontstyle = PANGO_STYLE_NORMAL;
+}
 
 void gp_cairo_set_linewidth(plot_struct *plot, double linewidth)
 {
@@ -345,12 +357,12 @@ void gp_cairo_set_linewidth(plot_struct *plot, double linewidth)
 	/* draw any open polygon set */
 	gp_cairo_end_polygon(plot);
 
-	if (linewidth < 0.25)	/* Admittedly arbitrary */
-	    linewidth = 0.25;
-	plot->linewidth = linewidth;
-
 	if (!strcmp(term->name,"pdfcairo"))
-		plot->linewidth *= 2;
+		linewidth *= 2;
+
+	if (linewidth < 0.20)	/* Admittedly arbitrary */
+	    linewidth = 0.20;
+	plot->linewidth = linewidth;
 
 }
 
@@ -389,6 +401,9 @@ void gp_cairo_draw_polygon(plot_struct *plot, int n, gpiPoint *corners)
 	/* begin by stroking any open path */
 	gp_cairo_stroke(plot);
 
+	if (n <= 0)
+	    return;
+
 	if (plot->polygons_saturate) {
 		int i;
 		path_item *path;
@@ -451,14 +466,15 @@ void gp_cairo_end_polygon(plot_struct *plot)
 
 	/* if there's only one polygon, draw it directly */
 	if (path->previous == NULL) {
-		FPRINTF((stderr,"processing one polygon\n"));
-		cairo_move_to(plot->cr, path->corners[0].x, path->corners[0].y);
-		for (i=1;i<path->n;++i)
-			cairo_line_to(plot->cr, path->corners[i].x, path->corners[i].y);
-		cairo_close_path(plot->cr);
-		plot->color = path->color;
-		gp_cairo_fill( plot, path->corners->style & 0xf, path->corners->style >> 4 );
-		cairo_fill(plot->cr);
+		if (path->n > 0) {
+		    cairo_move_to(plot->cr, path->corners[0].x, path->corners[0].y);
+		    for (i = 1; i < path->n; i++)
+			    cairo_line_to(plot->cr, path->corners[i].x, path->corners[i].y);
+		    cairo_close_path(plot->cr);
+		    plot->color = path->color;
+		    gp_cairo_fill( plot, path->corners->style & 0xf, path->corners->style >> 4 );
+		    cairo_fill(plot->cr);
+		}
 		free(path->corners);
 		free(path);
 		plot->polygon_path_last = NULL;
@@ -477,8 +493,8 @@ void gp_cairo_end_polygon(plot_struct *plot)
 	context_sav = plot->cr;
 	surface = cairo_surface_create_similar(cairo_get_target(plot->cr),
                                              CAIRO_CONTENT_COLOR_ALPHA,
-                                             plot->device_xmax*SCALE,
-                                             plot->device_ymax*SCALE);
+                                             plot->device_xmax*plot->upsampling_rate*SCALE,
+                                             plot->device_ymax*plot->upsampling_rate*SCALE);
 	context = cairo_create(surface);
 	cairo_set_operator(context,CAIRO_OPERATOR_SATURATE);
 	if (plot->antialiasing)
@@ -510,7 +526,7 @@ void gp_cairo_end_polygon(plot_struct *plot)
 		plot->color = path->color;
 		gp_cairo_fill( plot, path->corners->style & 0xf, path->corners->style >> 4 );
 		cairo_fill(plot->cr);
-		/* free the ressources, and go to the next point */
+		/* free the resources, and go to the next point */
 		free(path->corners);
 		path2 = path->previous;
 		free(path);
@@ -605,7 +621,8 @@ void gp_cairo_stroke(plot_struct *plot)
 	cairo_save(plot->cr);
 
 	if (plot->linetype == LT_NODRAW) {
-		cairo_set_operator(plot->cr, CAIRO_OPERATOR_XOR);
+		cairo_set_operator(plot->cr, CAIRO_OPERATOR_DEST);
+		lw = 0.0;
 
 	} else if (lt == LT_AXIS || plot->linestyle == GP_CAIRO_DOTS) {
 		/* Grid lines (lt 0) */
@@ -763,14 +780,14 @@ static gchar * gp_cairo_convert(plot_struct *plot, const char* string)
 }
 
 /*
- * The following #ifdef WIN32 section is all to work around a bug in
+ * The following #ifdef _WIN32 section is all to work around a bug in
  * the cairo/win32 backend for font rendering.  It has the effect of
  * testing for libfreetype support, and using that instead if possible.
  * Suggested by cairo developer Behdad Esfahbod.
  * Allin Cottrell suggests that this not necessary anymore for newer
  * versions of cairo.
  */
-#if defined(WIN32) && (CAIRO_VERSION_MAJOR < 2) && (CAIRO_VERSION_MINOR < 10)
+#if defined(_WIN32) && (CAIRO_VERSION_MAJOR < 2) && (CAIRO_VERSION_MINOR < 10)
 static PangoLayout *
 gp_cairo_create_layout(cairo_t *cr)
 {
@@ -792,7 +809,7 @@ gp_cairo_create_layout(cairo_t *cr)
 #endif
 
     layout = pango_layout_new(context);
-    g_object_unref(context);
+    g_clear_object (&context);
 
     return layout;
 }
@@ -804,12 +821,18 @@ gp_cairo_create_layout(cairo_t *cr)
 }
 #endif
 
+void gp_cairo_set_resolution(int dpi)
+{
+	pango_cairo_font_map_set_resolution(PANGO_CAIRO_FONT_MAP(pango_cairo_font_map_get_default()), dpi);
+}
+
 void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string,
 		    int *width, int *height)
 {
 	double x,y;
-	double arg = plot->text_angle * M_PI/180;
+	double arg;
 	double vert_just, delta, deltax, deltay;
+	double box_x, box_y;
 	PangoRectangle ink_rect;
 	PangoRectangle logical_rect;
 	PangoLayout *layout;
@@ -833,7 +856,7 @@ void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string,
 	if (!strcmp(plot->fontname,"Symbol")) {
 		FPRINTF((stderr,"Parsing a Symbol string\n"));
 		string_utf8 = gp_cairo_convert_symbol_to_unicode(plot, string);
-		strncpy(plot->fontname, gp_cairo_default_font(), sizeof(plot->fontname));
+		safe_strncpy(plot->fontname, gp_cairo_default_font(), sizeof(plot->fontname));
 		symbol_font_parsed = TRUE;
 	} else
 #endif /*MAP_SYMBOL*/
@@ -852,7 +875,7 @@ void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string,
 #ifdef MAP_SYMBOL
 	/* restore the Symbol font setting */
 	if (symbol_font_parsed)
-		strncpy(plot->fontname, "Symbol", sizeof(plot->fontname));
+		safe_strncpy(plot->fontname, "Symbol", sizeof(plot->fontname));
 #endif /*MAP_SYMBOL*/
 	pango_font_description_set_size (desc, (int) (plot->fontsize*PANGO_SCALE*plot->oversampling_scale) );
 
@@ -879,11 +902,9 @@ void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string,
 	vert_just = 0.5 * (float)(plot->fontsize * plot->oversampling_scale);
 	vert_just = baseline_offset - vert_just;
 
-	x = (double) x1;
-	y = (double) y1;
-
-	x -= vert_just * sin(arg);
-	y -= vert_just * cos(arg);
+	arg = plot->text_angle * M_PI/180;
+	x = (double)x1 - vert_just * sin(arg);
+	y = (double)y1 - vert_just * cos(arg);
 
 	delta = ((double)logical_rect.width/2) / PANGO_SCALE;
 
@@ -908,10 +929,11 @@ void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string,
 #endif /* helper point */
 
 	cairo_save (plot->cr);
+	cairo_translate(plot->cr, x, y);
+	cairo_rotate(plot->cr, -arg);
+
 	cairo_set_source_rgba(plot->cr, plot->color.r, plot->color.g, plot->color.b,
 				1. - plot->color.alpha);
-	cairo_move_to (plot->cr, x-0.5, y-0.5);
-	cairo_rotate(plot->cr, -arg);
 
 	/* Inform Pango to re-layout the text with the new transformation */
 	pango_cairo_update_layout (plot->cr, layout);
@@ -920,36 +942,44 @@ void gp_cairo_draw_text(plot_struct *plot, int x1, int y1, const char* string,
 	 * Do it by ourselves, or we can get spurious lines on future calls. */
 	cairo_new_path(plot->cr);
 
-#ifdef EAM_BOXED_TEXT
-	cairo_set_line_width(plot->cr, plot->linewidth*plot->oversampling_scale);
-	cairo_rotate(plot->cr, arg);
-	cairo_translate(plot->cr, x, y);
-	cairo_rotate(plot->cr, -arg);
+	if (in_textbox) {
+		box_rotation = -arg;
+		box_origin_x = x1;
+		box_origin_y = y1;
+		box_y = box_origin_y - vert_just;
+		switch (plot->justify_mode) {
+		case LEFT:
+			box_x = box_origin_x;
+			break;
+		default:
+		case CENTRE:
+			box_x = box_origin_x - delta;
+			break;
+		case RIGHT:
+			box_x = box_origin_x - 2*delta;
+			break;
+		}
 
-	{
-	PangoRectangle ink, logical;
-	pango_layout_get_pixel_extents (layout, &ink, &logical);
+		/* Update bounding box for boxed label text */
+		pango_layout_get_pixel_extents (layout, &ink_rect, &logical_rect);
 
-	/* Auto-initialization */
-	if (bounding_box[0] < 0 && bounding_box[1] < 0) {
-	    bounding_box[0] = bounding_box[2] = x;
-	    bounding_box[1] = bounding_box[3] = y;
+		/* Auto-initialization */
+		if (bounding_box[0] < 0 && bounding_box[1] < 0) {
+		    bounding_box[0] = bounding_box[2] = box_x;
+		    bounding_box[1] = bounding_box[3] = box_y;
+		}
+		if (bounding_box[0] > box_x + ink_rect.x)
+		    bounding_box[0] = box_x + ink_rect.x;
+		if (bounding_box[2] < box_x + ink_rect.x + ink_rect.width)
+		    bounding_box[2] = box_x + ink_rect.x + ink_rect.width;
+		if (bounding_box[1] > box_y + ink_rect.y)
+		    bounding_box[1] = box_y + ink_rect.y;
+		if (bounding_box[3] < box_y + ink_rect.y + ink_rect.height)
+		    bounding_box[3] = box_y + ink_rect.y + ink_rect.height;
 	}
 
-	/* Would it look better to use logical bounds rather than ink? */
-	if (bounding_box[0] > x + ink.x)
-	    bounding_box[0] = x + ink.x;
-	if (bounding_box[2] < x + ink.x + ink.width)
-	    bounding_box[2] = x + ink.x + ink.width;
-	if (bounding_box[1] > y + ink.y)
-	    bounding_box[1] = y + ink.y;
-	if (bounding_box[3] < y + ink.y + ink.height)
-	    bounding_box[3] = y + ink.y + ink.height;
-	}
-#endif
-
 	/* free the layout object */
-	g_object_unref (layout);
+	g_clear_object (&layout);
 	cairo_restore (plot->cr);
 }
 
@@ -1092,7 +1122,7 @@ void gp_cairo_draw_point(plot_struct *plot, int x1, int y1, int style)
 		if (style == 14)
 			cairo_fill_preserve(plot->cr);
 		cairo_stroke(plot->cr);
-		break;				
+		break;
 	default :
 		break;
 	}
@@ -1289,11 +1319,11 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		enhanced_text_utf8 = gp_cairo_convert_symbol_to_unicode(plot, gp_cairo_enhanced_string);
 
 		if (!strcmp(plot->fontname,"Symbol")) {
-			strncpy(gp_cairo_enhanced_font,
+			safe_strncpy(gp_cairo_enhanced_font,
 				plot->fontname,
 				sizeof(gp_cairo_enhanced_font));
 		} else {
-			strncpy(gp_cairo_enhanced_font,
+			safe_strncpy(gp_cairo_enhanced_font,
 				gp_cairo_default_font(), sizeof(gp_cairo_enhanced_font));
 		}
 		symbol_font_parsed = TRUE;
@@ -1318,8 +1348,11 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		pango_layout_set_text (save_layout, gp_cairo_save_utf8, -1);
 		pango_layout_set_attributes (save_layout, gp_cairo_enhanced_save_AttrList);
 		pango_layout_get_extents(save_layout, NULL, &save_logical_rect);
-		g_object_unref (save_layout);
+		g_clear_object (&save_layout);
+
 		pango_attr_list_unref( gp_cairo_enhanced_save_AttrList );
+		gp_cairo_enhanced_save_AttrList = NULL;
+
 		/* invert the size, so we will go back to the saved state */
 		save_logical_rect.width = -save_logical_rect.width;
 		/* EAM FIXME:  Zero height necessary but I don't understand why */
@@ -1327,7 +1360,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		/* adding a blank character with the corresponding shape */
 		gp_cairo_add_shape(save_logical_rect,start);
 
-		strncpy(gp_cairo_save_utf8, "", sizeof(gp_cairo_save_utf8));
+		safe_strncpy(gp_cairo_save_utf8, "", sizeof(gp_cairo_save_utf8));
 		gp_cairo_enhanced_restore_now = FALSE;
 		start++;
 	}
@@ -1346,7 +1379,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		else
 			pango_layout_set_attributes (underprinted_layout, gp_cairo_enhanced_underprinted_AttrList);
 		pango_layout_get_extents(underprinted_layout, NULL, &underprinted_logical_rect);
-		g_object_unref (underprinted_layout);
+		g_clear_object (&underprinted_layout);
 
 		/* compute the size of the text to overprint*/
 
@@ -1363,7 +1396,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		pango_layout_set_font_description (current_layout, current_desc);
 		pango_font_description_free (current_desc);
 		pango_layout_get_extents(current_layout, &current_ink_rect, &current_logical_rect);
-		g_object_unref (current_layout);
+		g_clear_object (&current_layout);
 
 		/* calculate the distance to remove to center the overprinted text */
 		underprinted_logical_rect.width = -(underprinted_logical_rect.width + current_logical_rect.width)/2;
@@ -1372,7 +1405,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		/* adding a blank character with the corresponding shape */
 		gp_cairo_add_shape(underprinted_logical_rect, start);
 
-		strncpy(gp_cairo_underprinted_utf8, "", sizeof(gp_cairo_underprinted_utf8));
+		safe_strncpy(gp_cairo_underprinted_utf8, "", sizeof(gp_cairo_underprinted_utf8));
 		/* increment the position as we added a character */
 		start++;
 	}
@@ -1392,7 +1425,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		pango_layout_set_text (current_layout, gp_cairo_utf8, -1);
 		pango_layout_set_attributes (current_layout, gp_cairo_enhanced_AttrList);
 		pango_layout_get_extents(current_layout, &current_ink_rect, &current_logical_rect);
-		g_object_unref (current_layout);
+		g_clear_object (&current_layout);
 
 		/* we first compute the size of the text */
 		/* Create a PangoLayout, set the font and text */
@@ -1408,7 +1441,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		pango_font_description_free (hide_desc);
 
 		pango_layout_get_extents(hide_layout, &hide_ink_rect, &hide_logical_rect);
-		g_object_unref (hide_layout);
+		g_clear_object (&hide_layout);
 
 		/* rect.y must be reworked to take previous text into account, which may be smaller */
 		/* hide_logical_rect.y is always initialized at zero, but should be : */
@@ -1437,7 +1470,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 		pango_layout_set_font_description (zerowidth_layout, zerowidth_desc);
 		pango_font_description_free (zerowidth_desc);
 		pango_layout_get_extents(zerowidth_layout, NULL, &zerowidth_logical_rect);
-		g_object_unref (zerowidth_layout);
+		g_clear_object (&zerowidth_layout);
 
 		/* invert the size, so we will go back to the start of the string */
 		zerowidth_logical_rect.width = -zerowidth_logical_rect.width;
@@ -1482,7 +1515,7 @@ void gp_cairo_enhanced_flush(plot_struct *plot)
 
 #ifdef MAP_SYMBOL
 	if (symbol_font_parsed)
-		strncpy(gp_cairo_enhanced_font, "Symbol", sizeof(gp_cairo_enhanced_font));
+		safe_strncpy(gp_cairo_enhanced_font, "Symbol", sizeof(gp_cairo_enhanced_font));
 #endif /* MAP_SYMBOL */
 
 	g_free(enhanced_text_utf8);
@@ -1518,11 +1551,12 @@ void gp_cairo_enhanced_open(plot_struct *plot, char* fontname, double fontsize,
 	}
 
 	if (!gp_cairo_enhanced_opened_string) {
-		// Strip off Bold or Italic and apply immediately
-		// Is it really necessary to preserve plot->fontname?
+		/* Strip off Bold or Italic and apply immediately
+		 * Is it really necessary to preserve plot->fontname?
+		 */
 		char *save_plot_font = strdup(plot->fontname);
 		gp_cairo_set_font(plot, fontname, plot->fontsize);
-		strncpy(gp_cairo_enhanced_font, plot->fontname, sizeof(gp_cairo_enhanced_font));
+		safe_strncpy(gp_cairo_enhanced_font, plot->fontname, sizeof(gp_cairo_enhanced_font));
 		strcpy(plot->fontname, save_plot_font);
 		free(save_plot_font);
 
@@ -1555,7 +1589,7 @@ void gp_cairo_enhanced_init(plot_struct *plot, int len)
 	gp_cairo_enhanced_overprint = FALSE;
 	gp_cairo_enhanced_showflag = TRUE;
 	gp_cairo_enhanced_fontsize = plot->fontsize*plot->oversampling_scale;
-	strncpy(gp_cairo_enhanced_font, plot->fontname, sizeof(gp_cairo_enhanced_font));
+	safe_strncpy(gp_cairo_enhanced_font, plot->fontname, sizeof(gp_cairo_enhanced_font));
 	gp_cairo_enhanced_AttrList = pango_attr_list_new();
 }
 
@@ -1564,6 +1598,7 @@ void gp_cairo_enhanced_finish(plot_struct *plot, int x, int y)
 	PangoRectangle ink_rect, logical_rect;
 	PangoLayout *layout;
 	double vert_just, arg, enh_x, enh_y, delta, deltax, deltay;
+	double box_x, box_y;
 	int baseline_offset;
 
 	/* Create a PangoLayout, set the font and text */
@@ -1579,7 +1614,7 @@ void gp_cairo_enhanced_finish(plot_struct *plot, int x, int y)
 	baseline_offset = pango_layout_get_baseline(layout) / PANGO_SCALE;
 	vert_just = 0.5 * (float)(plot->fontsize * plot->oversampling_scale);
 	vert_just = baseline_offset - vert_just;
-	
+
 	arg = plot->text_angle * M_PI/180;
 	enh_x = x - vert_just * sin(arg);
 	enh_y = y - vert_just * cos(arg);
@@ -1603,8 +1638,7 @@ void gp_cairo_enhanced_finish(plot_struct *plot, int x, int y)
 	}
 
 	cairo_save(plot->cr);
-	cairo_move_to (plot->cr, enh_x, enh_y);
-	/* angle in radians */
+	cairo_translate(plot->cr, enh_x, enh_y);
 	cairo_rotate(plot->cr, -arg);
 
 	cairo_set_source_rgba(plot->cr, plot->color.r, plot->color.g, plot->color.b,
@@ -1616,32 +1650,48 @@ void gp_cairo_enhanced_finish(plot_struct *plot, int x, int y)
 	 * Do it by ourselves, or we can get spurious lines on future calls. */
 	cairo_new_path(plot->cr);
 
-#ifdef EAM_BOXED_TEXT
-	/* Update bounding box for boxed label text */
-	pango_layout_get_pixel_extents (layout, &ink_rect, &logical_rect);
+	if (in_textbox) {
+		box_rotation = -arg;
+		box_origin_x = x;
+		box_origin_y = y;
+		box_y = box_origin_y - vert_just;
+		switch (plot->justify_mode) {
+		case LEFT:
+			box_x = box_origin_x;
+			break;
+		default:
+		case CENTRE:
+			box_x = box_origin_x - delta;
+			break;
+		case RIGHT:
+			box_x = box_origin_x - 2*delta;
+			break;
+		}
 
-	/* Auto-initialization */
-	if (bounding_box[0] < 0 && bounding_box[1] < 0) {
-	    bounding_box[0] = bounding_box[2] = x;
-	    bounding_box[1] = bounding_box[3] = y;
+		/* Update bounding box for boxed label text */
+		pango_layout_get_pixel_extents (layout, &ink_rect, &logical_rect);
+
+		/* Auto-initialization */
+		if (bounding_box[0] < 0 && bounding_box[1] < 0) {
+		    bounding_box[0] = bounding_box[2] = box_x;
+		    bounding_box[1] = bounding_box[3] = box_y;
+		}
+		if (bounding_box[0] > box_x + ink_rect.x)
+		    bounding_box[0] = box_x + ink_rect.x;
+		if (bounding_box[2] < box_x + ink_rect.x + ink_rect.width)
+		    bounding_box[2] = box_x + ink_rect.x + ink_rect.width;
+		if (bounding_box[1] > box_y + ink_rect.y)
+		    bounding_box[1] = box_y + ink_rect.y;
+		if (bounding_box[3] < box_y + ink_rect.y + ink_rect.height)
+		    bounding_box[3] = box_y + ink_rect.y + ink_rect.height;
 	}
 
-	/* Would it look better to use logical bounds rather than ink_rect? */
-	if (bounding_box[0] > enh_x + ink_rect.x)
-	    bounding_box[0] = enh_x + ink_rect.x;
-	if (bounding_box[2] < enh_x + ink_rect.x + ink_rect.width)
-	    bounding_box[2] = enh_x + ink_rect.x + ink_rect.width;
-	if (bounding_box[1] > enh_y + ink_rect.y)
-	    bounding_box[1] = enh_y + ink_rect.y;
-	if (bounding_box[3] < enh_y + ink_rect.y + ink_rect.height)
-	    bounding_box[3] = enh_y + ink_rect.y + ink_rect.height;
-#endif
-	
 	/* free the layout object */
 	pango_attr_list_unref( gp_cairo_enhanced_AttrList );
-	g_object_unref (layout);
+	gp_cairo_enhanced_AttrList = NULL;
+	g_clear_object (&layout);
 	cairo_restore(plot->cr);
-	strncpy(gp_cairo_utf8, "", sizeof(gp_cairo_utf8));
+	safe_strncpy(gp_cairo_utf8, "", sizeof(gp_cairo_utf8));
 	free(gp_cairo_enhanced_string);
 }
 
@@ -1699,7 +1749,6 @@ void gp_cairo_fill(plot_struct *plot, int fillstyle, int fillpar)
 	}
 }
 
-#ifdef EAM_BOXED_TEXT
 void gp_cairo_boxed_text(plot_struct *plot, int x, int y, int option)
 {
 	int dx, dy;
@@ -1709,6 +1758,7 @@ void gp_cairo_boxed_text(plot_struct *plot, int x, int y, int option)
 		/* Initialize bounding box for this text string */
 		bounding_box[0] = bounding_box[2] = x;
 		bounding_box[1] = bounding_box[3] = y;
+		in_textbox = TRUE;
 		break;
 
 	case TEXTBOX_OUTLINE:
@@ -1721,30 +1771,44 @@ void gp_cairo_boxed_text(plot_struct *plot, int x, int y, int option)
 		gp_cairo_end_polygon(plot);
 
 		cairo_save(plot->cr);
-		dx = 0.5 * bounding_xmargin * (float)(plot->fontsize * plot->oversampling_scale);
-		dy = 0.5 * bounding_ymargin * (float)(plot->fontsize * plot->oversampling_scale);
+
+		/* In progress: textbox rotation
+		 * 1) translate to text origin
+		 * 2) subtract translation from bounding box
+		 * 3) rotate about new origin
+		 * 4) stroke/fill
+		 */
+		cairo_translate(plot->cr, box_origin_x, box_origin_y);
+		cairo_rotate(plot->cr, box_rotation);
+		cairo_translate(plot->cr, -box_origin_x, -box_origin_y);
+
+		dx = 0.25 * bounding_xmargin * (float)(plot->fontsize * plot->oversampling_scale);
+		dy = 0.25 * bounding_ymargin * (float)(plot->fontsize * plot->oversampling_scale);
 		if (option == TEXTBOX_GREY)
 		    dy = 0;
-		gp_cairo_move(plot,   bounding_box[0]-dx, bounding_box[1]-dy); 
-		gp_cairo_vector(plot, bounding_box[0]-dx, bounding_box[3]+dy); 
-		gp_cairo_vector(plot, bounding_box[2]+dx, bounding_box[3]+dy); 
-		gp_cairo_vector(plot, bounding_box[2]+dx, bounding_box[1]-dy); 
-		gp_cairo_vector(plot, bounding_box[0]+dx, bounding_box[1]-dy); 
+		gp_cairo_move(plot,   bounding_box[0]-dx, bounding_box[1]-dy);
+		gp_cairo_vector(plot, bounding_box[0]-dx, bounding_box[3]+dy);
+		gp_cairo_vector(plot, bounding_box[2]+dx, bounding_box[3]+dy);
+		gp_cairo_vector(plot, bounding_box[2]+dx, bounding_box[1]-dy);
+		gp_cairo_vector(plot, bounding_box[0]+dx, bounding_box[1]-dy);
 		cairo_close_path(plot->cr);
 		if (option == TEXTBOX_BACKGROUNDFILL) {
-		    rgb_color *background = &gp_cairo_colorlist[0];
-		    cairo_set_source_rgb(plot->cr, background->r, background->g, background->b);
+		    cairo_set_source_rgba(plot->cr, plot->color.r, plot->color.g,
+					  plot->color.b, 1. - plot->color.alpha);
 		    cairo_fill(plot->cr);
 		} else if (option == TEXTBOX_GREY) {
 		    cairo_set_source_rgba(plot->cr, 0.75, 0.75, 0.75, 0.50);
 		    cairo_fill(plot->cr);
-		} else {
-		    cairo_set_line_width(plot->cr, 1.0*plot->oversampling_scale);
-		    cairo_set_source_rgb(plot->cr,
-			plot->color.r, plot->color.g, plot->color.b);
+		} else {  /* option == TEXTBOX_OUTLINE */
+		    cairo_set_line_width(plot->cr,
+					 plot->linewidth*plot->oversampling_scale);
+		    cairo_set_source_rgba(plot->cr, plot->color.r, plot->color.g,
+					  plot->color.b, 1. - plot->color.alpha);
 		    cairo_stroke(plot->cr);
 		}
+
 		cairo_restore(plot->cr);
+		in_textbox = FALSE;
 		break;
 
 	case TEXTBOX_MARGINS: /* Change the margin between text and box */
@@ -1756,7 +1820,6 @@ void gp_cairo_boxed_text(plot_struct *plot, int x, int y, int option)
 		break;
 	}
 }
-#endif
 
 #define PATTERN_SIZE 8
 
@@ -1872,6 +1935,7 @@ void gp_cairo_set_termvar(plot_struct *plot, unsigned int *v_char,
 	PangoRectangle ink_rect;
 	PangoRectangle logical_rect;
 	unsigned int tmp_v_char, tmp_h_char;
+	extern int debug;
 
 	/* Create a PangoLayout, set the font and text */
 	layout = gp_cairo_create_layout (plot->cr);
@@ -1885,7 +1949,7 @@ void gp_cairo_set_termvar(plot_struct *plot, unsigned int *v_char,
 	pango_layout_set_font_description (layout, desc);
 	pango_font_description_free (desc);
 	pango_layout_get_extents(layout, &ink_rect, &logical_rect);
-	g_object_unref (layout);
+	g_clear_object (&layout);
 
 	/* we don't use gnuplot_x() and gnuplot_y() in the following
 	 * as the scale should have just been updated to 1.
@@ -1894,6 +1958,16 @@ void gp_cairo_set_termvar(plot_struct *plot, unsigned int *v_char,
 	tmp_v_char = (int) ceil( (double) logical_rect.height/PANGO_SCALE) - 1;
 	tmp_h_char = (int) ceil( (double) logical_rect.width/(10*PANGO_SCALE));
 
+	/* Desperation fallback case.
+	 * There have been reports of failure to obtain font metrics.
+	 * We don't know why (pango use of harfbuzz is a suspect).
+	 */
+	if (tmp_v_char <= 1 || tmp_h_char <= 1 || debug == 7) {
+		tmp_h_char = 140 + (plot->fontsize - 10.) * 16.;
+		tmp_v_char = 300 * (plot->fontsize / 10.);
+		fprintf(stderr, "warning: problem determining pango font metrics\n");
+	}
+
 	if (v_char)
 		*v_char = tmp_v_char;
 	if (h_char)
@@ -1948,7 +2022,7 @@ const char* gp_cairo_enhanced_get_fontname(plot_struct *plot)
 const char *
 gp_cairo_default_font(void)
 {
-#ifdef WIN32
+#ifdef _WIN32
 	return "Tahoma";
 #else
 	return "Sans";
