@@ -1,7 +1,3 @@
-/*
- * $Id: wxt_gui.h,v 1.48.2.4 2014/12/26 23:23:03 sfeam Exp $
- */
-
 /* GNUPLOT - wxt_gui.h */
 
 /*[
@@ -77,7 +73,7 @@
 # include <wx/wx.h>
 #endif /* WX_PRECOMP */
 
-/* clipboard functionnality */
+/* clipboard functionality */
 #include <wx/dataobj.h>
 #include <wx/clipbrd.h>
 
@@ -102,28 +98,29 @@
 /* wxMemoryInputStream, for the embedded PNG icons */
 #include <wx/mstream.h>
 
+/* Debugging support, required to turn off asserts */
+#include <wx/debug.h>
+
+/* printer data */
+#include <wx/cmndata.h>
+
 /* c++ vectors and lists, used to store gnuplot commands */
 #include <vector>
 #include <list>
 
-/* suprisingly Cocoa version of wxWidgets does not define _Bool ! */
+/* surprisingly Cocoa version of wxWidgets does not define _Bool ! */
 #ifdef __WXOSX_COCOA__
 #define _Bool bool
 #endif
 
 extern "C" {
-/* for interactive */
-# include "plot.h"
-/* for stdfn.h, JUSTIFY, encoding, *term definition, color.h */
-# include "term_api.h"
-/* for do_event declaration */
-# include "mouse.h"
-/* for rgb functions */
-# include "getcolor.h"
-/* for paused_for_mouse, PAUSE_BUTTON1 and friends */
-# include "command.h"
-/* for int_error */
-# include "util.h"
+#include "plot.h"	/* for interactive */
+#include "term_api.h"	/* for stdfn.h, JUSTIFY, encoding, *term definition, color.h */
+#include "mouse.h"	/* for do_event declaration */
+#include "getcolor.h"	/* for rgb functions */
+#include "command.h"	/* for paused_for_mouse, PAUSE_BUTTON1 and friends */
+#include "util.h"	/* for int_error, locale */
+#include "readline.h"	/* for wrap_readline_signal_handler */
 }
 
 /* if the gtk headers are available, use them to tweak some behaviours */
@@ -141,7 +138,7 @@ extern "C" {
 # define IMAGE_SURFACE
 #endif
 
-/* temporarly undef GTK_SURFACE for two reasons :
+/* temporarily undef GTK_SURFACE for two reasons :
  * - because of a CAIRO_OPERATOR_SATURATE bug,
  * - because as for now, it is slower than the pure image surface,
  * (multiple copies between video memory and main memory for operations that are
@@ -163,15 +160,23 @@ extern "C" {
 #endif
 #endif
 
+/* Enable the print dialog on Windows only
+ */
+#ifdef __WXMSW__
+# ifndef WXT_PRINT
+#  define WXT_PRINT
+# endif
+#endif
+
 extern "C" {
 /* Windows native backend,
  * redefinition of fprintf, getch...
  * console window */
-# ifdef _Windows
-#  ifndef _WIN32_WINNT 
-#   define _WIN32_WINNT 0x0500
+# ifdef _WIN32
+#  ifndef _WIN32_WINNT
+#   define _WIN32_WINNT 0x0501
 #  endif
-#  include "Windows.h"
+#  include <windows.h>
 #  include "win/wtext.h"
 #  include "win/winmain.h"
 # endif
@@ -192,7 +197,13 @@ extern "C" {
 #  include <gtk/gtk.h>
 # endif
 
-# ifdef _Windows
+/* wxWidgets 3.0 -> 2.8 compatibility */
+#if (wxMAJOR_VERSION < 3)
+#define wxBRUSHSTYLE_SOLID wxSOLID
+#define wxPENSTYLE_SOLID wxSOLID
+#endif
+
+# ifdef _WIN32
 #  include <cairo-win32.h>
 # endif
 
@@ -237,15 +248,15 @@ public:
 static wxtThread * thread;
 #endif /* WXT_MULTITHREADED */
 
-DECLARE_EVENT_TYPE(wxExitLoopEvent, -1)
-DEFINE_EVENT_TYPE(wxExitLoopEvent)
+DECLARE_LOCAL_EVENT_TYPE(wxExitLoopEvent, -1)
+DEFINE_LOCAL_EVENT_TYPE(wxExitLoopEvent)
 
-DECLARE_EVENT_TYPE(wxCreateWindowEvent, -1)
-DEFINE_EVENT_TYPE(wxCreateWindowEvent)
+DECLARE_LOCAL_EVENT_TYPE(wxCreateWindowEvent, -1)
+DEFINE_LOCAL_EVENT_TYPE(wxCreateWindowEvent)
 
 #ifdef USE_MOUSE
-DECLARE_EVENT_TYPE(wxStatusTextEvent, -1)
-DEFINE_EVENT_TYPE(wxStatusTextEvent)
+DECLARE_LOCAL_EVENT_TYPE(wxStatusTextEvent, -1)
+DEFINE_LOCAL_EVENT_TYPE(wxStatusTextEvent)
 #endif /* USE_MOUSE */
 
 /* Define a new application type, each gui should derive a class from wxApp */
@@ -254,8 +265,8 @@ class wxtApp : public wxApp
 public:
 #if defined(WXT_MULTITHREADED) && defined(WX_NEEDS_XINITTHREADS) && defined(X11)
 	/* Magic fix needed by wxgtk3.0 */
-        wxtApp() : wxApp() { XInitThreads(); } 
-#endif 
+        wxtApp() : wxApp() { XInitThreads(); }
+#endif
 
 	/* This one is called just after wxWidgets initialization */
 	bool OnInit();
@@ -301,9 +312,7 @@ typedef enum wxt_gp_command_t {
 	command_image,
 	command_layer,
 	command_hypertext
-#ifdef EAM_BOXED_TEXT
 	,command_boxed_text
-#endif
 	,command_dashtype
 } wxt_gp_command_t;
 
@@ -401,6 +410,7 @@ public :
 	void wxt_cairo_refresh();
 	void wxt_cairo_exec_command(gp_command command);
 	void wxt_cairo_draw_hypertext();
+	void wxt_cairo_draw_hyperimage();
 
 	/* the plot structure, defined in gp_cairo.h */
 	plot_struct plot;
@@ -429,8 +439,6 @@ private:
 #if defined(GTK_SURFACE)
 	GdkPixmap *gdkpixmap;
 #elif defined(__WXMSW__)
-	HDC hdc;
-	HBITMAP hbm;
 #else /* generic 'image' surface */
 	unsigned int *data32;
 	wxBitmap* cairo_bitmap;
@@ -487,6 +495,9 @@ public:
 	void OnSize( wxSizeEvent& event );
 	void OnCopy( wxCommandEvent& event );
 	void OnExport( wxCommandEvent& event );
+#ifdef WXT_PRINT
+	void OnPrint( wxCommandEvent& event );
+#endif
 #ifdef USE_MOUSE
 	void OnReplot( wxCommandEvent& event );
 	void OnToggleGrid( wxCommandEvent& event );
@@ -510,7 +521,10 @@ public:
 
 private:
 	wxtConfigDialog * config_dialog;
-
+#ifdef WXT_PRINT
+	// persistent printer choice and settings
+	wxPrintData printData;
+#endif
 	/* any class wishing to process wxWidgets events must use this macro */
 	DECLARE_EVENT_TABLE()
 };
@@ -520,6 +534,9 @@ enum {
 /* start at wxID_HIGHEST to avoid collisions */
 Toolbar_CopyToClipboard = wxID_HIGHEST,
 Toolbar_ExportToFile,
+#ifdef WXT_PRINT
+Toolbar_Print,
+#endif
 Toolbar_Replot,
 Toolbar_ToggleGrid,
 Toolbar_ZoomPrevious,
@@ -593,7 +610,6 @@ static wxt_window_t* wxt_findwindowbyid(wxWindowID);
 
 /* pointers to currently active instances */
 static wxt_window_t *wxt_current_window;
-static command_list_t *wxt_current_command_list;
 static wxtPanel *wxt_current_panel;
 static plot_struct *wxt_current_plot;
 
@@ -638,6 +654,12 @@ static void wxt_sigint_init();
 static void wxt_sigint_restore();
 static int wxt_sigint_counter = 0;
 
+/* exception handling if int_error() is called inside event processing */
+#if defined(HAVE_GTK)
+   #include <setjmp.h>
+   extern JMP_BUF *wxt_env;
+#endif
+
 /* cleanup at exit, and handle 'persist' setting */
 void wxt_atexit();
 
