@@ -1,5 +1,3 @@
-/* $Id: driver.h,v 1.22 2006/10/08 21:11:08 sfeam Exp $ */
-
 /* GNUPLOT - driver.h */
 
 /*[
@@ -42,29 +40,23 @@
 
 /* functions provided by term.c */
 
-static void do_point __PROTO((unsigned int x, unsigned int y, int number));
-static void line_and_point __PROTO((unsigned int x, unsigned int y, int number));
-static int null_text_angle __PROTO((int ang));
-static int null_justify_text __PROTO((enum JUSTIFY just));
-static int null_scale __PROTO((double x, double y));
-static void options_null __PROTO((void));
-static void UNKNOWN_null __PROTO((void));
-/* static int set_font_null __PROTO((const char *s));     */ /* unused */
+static void do_point(unsigned int x, unsigned int y, int number);
+static int null_text_angle(float ang);
+static int null_justify_text(enum JUSTIFY just);
+static int null_scale(double x, double y);
+static void options_null(void);
+static void UNKNOWN_null(void);
+
 #define set_font_null NULL
+#if  defined(WITH_METAPOST) || defined(WITH_METAFONT) || defined(WITH_TEKTRONIX) || !defined(NO_BITMAP_SUPPORT)
+static void line_and_point(unsigned int x, unsigned int y, int number);
+#endif
 
 extern FILE *gpoutfile;
 extern struct termentry *term;
 
 /* for use by all drivers */
-#ifndef NEXT
 #define sign(x) ((x) >= 0 ? 1 : -1)
-#else
-/* it seems that sign as macro causes some conflict with precompiled headers */
-static int sign(int x)
-{
-    return x >= 0 ? 1 : -1;
-}
-#endif /* NEXT */
 
 /* abs as macro is now uppercase, there are conflicts with a few C compilers
    that have abs as macro, even though ANSI defines abs as function
